#! /usr/bin/env python
# -*- coding:utf-8 -*-

from influxdb import InfluxDBClient
import random


def getClient():
    client = InfluxDBClient("influxdb.navyran.top", 80, "", "", "mydb")  # 初始化
    return client



def showTables(client):
    result = client.query('show measurements;')  # 显示数据库中的表
    print("Result: {0}".format(result))


def test():
    client = InfluxDBClient("localhost", 8086, "", "", "")  # 初始化
    print(client.get_list_database())  # 显示所有数据库名称
    # client.create_database("testdb")  # 创建数据库
    # print(client.get_list_database())  # 显示所有数据库名称
    # client.drop_database("testdb")  # 删除数据库
    # print(client.get_list_database())  # 显示所有数据库名称


def insert(client):
    json_body = [
        {
            "measurement": "students",
            "tags": {
                "stuid": "s123"
            },
            # "time": "2017-03-12T22:00:00Z",
            "fields": {
                "score": random.randint(20,100)
            }
        }
    ]
    print(json_body)
    client.write_points(json_body)  # 写入数据，同时创建表


def dropTable(client, table):
    client.query("drop measurement " + table)  # 删除表


def query(client):
    # result = client.query('select * from students;')
    result = client.query('SELECT * FROM "stock" WHERE time >= 1541030294278ms;')
    print("Result: {0}".format(result))


def main():
    # test()
    client = getClient()
    # showDBNames(client)
    showTables(client)
    # insert(client)
    query(client)


if __name__ == '__main__':
    main()



import xlsxwriter, string, openpyxl, os, math
from pylib import collection_util, file_util


# arr:需要循环加字母的数组
# level：需要加的层级
def cycle_letter(arr, level):
    tempArr = []
    letterArr = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', \
                 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z']
    arrNum = len(arr)
    if (level == 0 or arrNum == 0):
        return letterArr
    for index in range(arrNum):
        for letter in letterArr:
            tempArr.append(arr[index] + letter)
    return tempArr


# arr:需要生成的Excel列名称数目
def reduce_excel_col_name(num):
    tempVal = 1
    level = 1
    while (tempVal):
        tempVal = num / (math.pow(26, level))
        if (tempVal > 1):
            level += 1
        else:
            break

    excelArr = []
    tempArr = []
    for index in range(level):
        tempArr = cycle_letter(tempArr, index)
        for numIndex in range(len(tempArr)):
            if (len(excelArr) < num):
                excelArr.append(tempArr[numIndex])
            else:
                return excelArr
    return excelArr


def write_list_list_to_csv(list_list, file, titles=[], encoding='utf-8'):
    output = open(file, 'w', encoding=encoding)
    if len(titles) > 0:
        titlestr = '\t'.join(titles)
        output.write(titlestr + '\n')
    for i in range(len(list_list)):
        for j in range(len(list_list[i])):
            output.write(str(list_list[i][j]))  # write函数不能写int类型的参数，所以使用str()转化
            output.write('\t')  # 相当于Tab一下，换一个单元格
        output.write('\n')  # 写完一行立马换行
    output.close()


# data=[[],[]]
def write_list_list_to_excel(data, file, titles=[], start_row=1, encoding='utf-8'):
    print('data:', data)
    workbook = xlsxwriter.Workbook(file, {'nan_inf_to_errors': True})
    worksheet = workbook.add_worksheet()

    # Add a bold format to use to highlight cells.
    bold = workbook.add_format({'bold': 1})

    # letter_list = [chr(x) for x in range(ord('A'), ord('Z') + 1)]
    letter_list = reduce_excel_col_name(len(titles))
    print('letter_list:', letter_list)

    for i, title in enumerate(titles):
        # Write some data headers.
        worksheet.write(letter_list[i] + '1', title, bold)

    # Write some data to add to plot on the chart.
    # data = [
    #     [1, 2, 3, 4, 5],
    #     [2, 4, 6, 8, 10],
    #     [3, 6, 9, 12, 15],
    # ]

    for i, title in enumerate(titles):
        # 按列插入
        print(i, title)
        print(data[i])
        worksheet.write_column(row=start_row + 1, col=i, data=data[i])

    # Create a new Chart object.
    # chart = workbook.add_chart({'type': 'column'})
    # Configure the chart. In simplest case we add one or more data series.
    # chart.add_series({'values': '=Sheet1!$A$1:$A$5'})
    # chart.add_series({'values': '=Sheet1!$B$1:$B$5'})
    # chart.add_series({'values': '=Sheet1!$C$1:$C$5'})

    # Insert the chart into the worksheet.
    # worksheet.insert_chart('A7', chart)

    workbook.close()


def append_dict_list_to_excel(data, file, titles=[], encoding='utf-8'):
    if not os.path.exists(file):
        write_dict_list_to_excel(data, file, titles, 1)
    # 以只读模式读取
    wb = openpyxl.load_workbook(file, read_only=True)
    # 新建一个工作薄
    wb = openpyxl.Workbook()

    # 获取当前正在使用的sheet页
    ws = wb.active
    # 获取所有行或列，返回generator
    rows = ws.rows
    columns = ws.columns
    index = len(rows)

    write_dict_list_to_excel(data, file, titles, index)


def write_dict_list_to_excel(data, file, titles=[], start_row=1, encoding='utf-8'):
    print('titles:', titles)
    list = collection_util.convert_dict_list_to_list_list(data, titles)
    print("list:", list)
    write_list_list_to_excel(list, file, titles, start_row, encoding)


def export_to_html_table(table_title, colume_title=[], data_list=[]):
    from HTMLTable import (
        HTMLTable,
    )

    # 标题
    table = HTMLTable(caption=table_title)

    title_tuple = (tuple(colume_title),)
    print(title_tuple)
    # 表头行
    table.append_header_rows(title_tuple)

    # list = [['荔枝', 11, 1, 10],['荔枝2', 11, 1, 10]]
    # tp = collection_util.convert_list_list_to_tuple_tuple(list)
    # print(tp)

    tps = collection_util.convert_list_list_to_tuple_tuple(data_list)
    print(tps)

    # 数据行
    table.append_data_rows(tps)

    # 标题样式
    table.caption.set_style({
        'font-size': '15px',
    })

    # 表格样式，即<table>标签样式
    table.set_style({
        'border-collapse': 'collapse',
        'word-break': 'keep-all',
        'white-space': 'nowrap',
        'font-size': '14px',
    })

    # 统一设置所有单元格样式，<td>或<th>
    table.set_cell_style({
        'border-color': '#000',
        'border-width': '1px',
        'border-style': 'solid',
        'padding': '5px',
    })

    # 表头样式
    table.set_header_row_style({
        'color': '#fff',
        'background-color': '#48a6fb',
        'font-size': '18px',
    })

    # 覆盖表头单元格字体样式
    table.set_header_cell_style({
        'padding': '15px',
    })

    # 调小次表头字体大小
    table[1].set_cell_style({
        'padding': '8px',
        'font-size': '15px',
    })

    # 遍历数据行，如果增长量为负，标红背景颜色
    for row in table.iter_data_rows():
        if row[2].value < 0:
            row.set_style({
                'background-color': '#ffdddd',
            })

    html = table.to_html()
    print(html)
    file_util.write_file(table_title + '.html', html)


if __name__ == '__main__':
    # write_list_list_to_excel([], 'test.xlsx', titles=['a', 'b', 'c'])
    data = [['荔枝', 11, 1, 10],
            ['芒果', 9, -1, -10],
            ['香蕉', 6, 1, 20]]
    export_to_html_table('果园', ['名称', '产量 (吨)', '环比', ''], data)



from bs4 import BeautifulSoup


# 匹配出所有标记，并返回标记的结果属性，
def match_tag_attr_list(html, html_tag, ret_tag_attr=None):
    soup = BeautifulSoup(html, 'html5lib')
    tags = soup.findAll(html_tag)
    list = []
    for t in tags:
        if ret_tag_attr:
            list.append(t.get(ret_tag_attr))
        else:
            list.append(t.contents)
            # print(t.get(ret_tag_attr))
            # print(t.contents)
    return list

# 返回html中所有表格
def match_table(html):
    soup = BeautifulSoup(html, 'html5lib')
    table = soup.findAll('table')
    print(table)

# 匹配到对象列表
def match_tag_list_by_xpath(html, xpath):
    soup = BeautifulSoup(html, 'lxml')
    # print(html)
    # xpath = 'ul.sellListContent > li.clear.LOGCLICKDATA'
    lis = soup.select(xpath)
    # for li in lis:
    #     print(li)
    return lis

# 返回匹配到的第一个tag对象
def match_tag_first_by_xpath(html, xpath):
    list = match_tag_list_by_xpath(html, xpath)
    if list:
        return list[0]
    return None



import pymysql

pymysql.install_as_MySQLdb()
import records
from . import url_util


def init_mysql_url(host, username, password, port, db):
    url = 'mysql://{}:{}@{}:{}/{}?charset=utf8'.format(
        url_util.urlencode(username), url_util.urlencode(password), host, port, db)
    return url


def conn(dburl):
    db = records.Database(
        dburl,
        encoding="utf-8", echo=False)
    return db


def exe_sql(db_conn, sql):
    rows = db_conn.query(sql)
    return rows


def query(url, sql, echo=False):
    db = records.Database(url, encoding="utf-8", echo=False)
    rows = db.query(sql)
    return rows


# 将record的结果转为[{}]
def record_result_to_dict_list(rows):
    ret = []
    for row in rows:
        keys = row.keys()
        values = list(row.values())

        dic = dict(zip(keys, values))
        # print(dic)
        ret.append(dic)
    return ret



#!-*-encoding:utf-8-*-

import redis
import json
import types
import collections
from threading import Thread
import time
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor

"""
"""

prodcons_queue_one = 'task:prodcons:queue:one'
prodcons_queue_two = 'task:prodcons:queue:two'
prodcons_queue_three = 'task:prodcons:queue:three'

queue_lst = [prodcons_queue_one, prodcons_queue_two, prodcons_queue_three]


class Task(object):
    def __init__(self):
        self.rcon = redis.StrictRedis(host="localhost", db=5)
        self.queue_lst = queue_lst

    def listen_task(self):
        while True:
            channel, task = self.rcon.blpop(self.queue_lst, 0)
            print("Task get", channel, task)


class MQServer(object):
    def __init__(self, conn_pool, channel_lst=[], batch_size=5, max_worker_num=10, pool_type='proc'):
        self.rcon = redis.Redis(connection_pool=conn_pool)
        self.process_pool = ProcessPoolExecutor(max_worker_num) if pool_type == 'proc' else ThreadPoolExecutor(
            max_worker_num)
        self.batch_size = batch_size
        self.channel_lst = channel_lst
        self.registed_handler_dic = collections.defaultdict(list)

    def regist_handler(self, channel, handler):
        if channel not in self.registed_handler_dic:
            self.channel_lst.append(channel)

        self.registed_handler_dic[channel].append(handler)

    def notify(self, channel, payload):
        channel = channel.decode('utf-8') if not isinstance(channel, str) else channel
        idx = 0
        for handler in self.registed_handler_dic[channel]:
            if isinstance(handler, types.FunctionType) or hasattr(handler, '__call__'):
                # print("取出handle")
                # 将handler 和数据封装到一个进程任务中
                self.process_pool.submit(handler, payload)

    def event_loop(self):
        """
        服务器以阻塞的方式运行
        :return:
        """
        while True:
            channel, payload = self.rcon.blpop(self.channel_lst)  # 阻塞方式
            # print(channel, payload)
            if not payload:
                continue
            self.notify(channel, payload)

    def batch_event_loop(self):
        """
        使用批次处理
        :return:
        """
        while True:
            for channel in self.channel_lst:
                cur_queue = self.rcon.lrange(channel, 0, self.batch_size - 1)

                cur_queue_len = len(cur_queue)
                if cur_queue_len == 0:
                    continue

                print("当前取出队列的长度为: %d" % cur_queue_len)
                # 从队列中移除当前处理的数据
                self.rcon.ltrim(channel, cur_queue_len, -1)

                payload = cur_queue
                self.notify(channel, payload)

    def close(self):
        self.pubsub.close()
        self.rcon.reset()
        self.process_pool.shutdown()


# 这里定义handler 函数和可执行类
interval = 10


def channel_one_handle_one(data):
    """
    :param data:
    :return:
    """
    print("开启进程handle 1 处理channel 1 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_two_handle_one(data):
    print("开启进程handle 1 处理channel 2 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_three_handle_one(data):
    print("开启进程handle 1 处理channel 3 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_one_handle_two(data):
    """
    :param data:
    :return:
    """
    print("开启进程handle 2 处理channel 1 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_two_handle_two(data):
    print("开启进程handle 2 处理channel 2 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_three_handle_two(data):
    print("开启进程handle 2 处理channel 3 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


class Channel_one_handler_three(object):
    def __init__(self):
        pass

    def __call__(self, data):
        print("开启进程handle 类 3 处理channel 1 的信息")
        time.sleep(interval)
        print("返回结果:")
        print(data)
        return


class Channel_two_handler_three(object):
    def __init__(self):
        pass

    def __call__(self, data):
        print("开启进程handle 类 3 处理channel 2 的信息")
        time.sleep(interval)
        print("返回结果:")
        print(data)
        return


def run_server():
    # ===================== params ==================================
    host = "localhost"
    port = 6379
    db = 5
    passwd = None

    # 信道
    prodcons_queue_one = 'task:prodcons:queue:one'
    prodcons_queue_two = 'task:prodcons:queue:two'
    prodcons_queue_three = 'task:prodcons:queue:three'

    # =================================================================
    rcon_pool = redis.ConnectionPool(host=host, port=port, db=db, password=passwd)

    mq_server = MQServer(rcon_pool, pool_type='th')

    mq_server.regist_handler(prodcons_queue_one, channel_one_handle_one)
    mq_server.regist_handler(prodcons_queue_one, channel_one_handle_two)
    # mq_server.regist_handler(prodcons_queue_one, Channel_one_handler_three())

    mq_server.regist_handler(prodcons_queue_two, channel_two_handle_one)
    mq_server.regist_handler(prodcons_queue_two, channel_two_handle_two)
    # mq_server.regist_handler(prodcons_queue_two, Channel_two_handler_three())

    mq_server.regist_handler(prodcons_queue_three, channel_three_handle_one)
    mq_server.regist_handler(prodcons_queue_three, channel_three_handle_two)

    try:
        # mq_server.event_loop()
        mq_server.batch_event_loop()
    except InterruptedError as e:
        mq_server.close()


if __name__ == '__main__':
    # print('listen task queue')
    # Task().listen_task()

    run_server()



import string, re, json, uuid
import random, math
from python_utils import converters


def convert_to_int(str):
    return converters.to_int(str)


# 编码转换
def convert_utf8(str, charset):
    str = str.decode(charset).encode('utf-8')  # 以gbk编码格式读取g（因为他就是gbk编码的）并转换为utf-8格式输出
    return str


# 判断字符是否为中文
def is_zh(c):
    x = ord(c)
    # Punct & Radicals
    if x >= 0x2e80 and x <= 0x33ff:
        return True

        # Fullwidth Latin Characters
    elif x >= 0xff00 and x <= 0xffef:
        return True

        # CJK Unified Ideographs &
    # CJK Unified Ideographs Extension A
    elif x >= 0x4e00 and x <= 0x9fbb:
        return True
        # CJK Compatibility Ideographs
    elif x >= 0xf900 and x <= 0xfad9:
        return True

        # CJK Unified Ideographs Extension B
    elif x >= 0x20000 and x <= 0x2a6d6:
        return True

        # CJK Compatibility Supplement
    elif x >= 0x2f800 and x <= 0x2fa1d:
        return True

    else:
        return False


# 拆分中文和应为到两个数组中
def split_zh_en(zh_en_str):
    zh_en_group = []
    zh_gather = ""
    en_gather = ""
    zh_status = False
    mark = {"en": 1, "zh": 2}

    for c in zh_en_str:
        if not zh_status and is_zh(c):
            zh_status = True
            if en_gather != "":
                zh_en_group.append([mark["en"], en_gather])
                en_gather = ""
        elif not is_zh(c) and zh_status:
            zh_status = False
            if zh_gather != "":
                zh_en_group.append([mark["zh"], zh_gather])
        if zh_status:
            zh_gather += c
        else:
            en_gather += c
            zh_gather = ""

    if en_gather != "":
        zh_en_group.append([mark["en"], en_gather])
    elif zh_gather != "":
        zh_en_group.append([mark["zh"], zh_gather])

    return zh_en_group


# 下划线转驼峰
def underline_to_camel(underline_format):
    word_set = underline_format.split('_')
    if len(word_set) == 1:
        return underline_format

    camel_format = ''
    for i in range(len(word_set)):
        if i == 0:
            continue
        word_set[i] = word_set[i].capitalize()
    return ''.join(word_set)


@classmethod
def ltrim(cls, haystack, left=""):
    if cls.isEmpty(haystack):
        return haystack
    elif cls.isEmpty(left):
        return haystack.lstrip()
    else:
        return haystack.lstrip(left)


# 下划线转为驼峰格式
@classmethod
def camelize(cls, uncamelized, separator="_"):
    uncamelized = separator + uncamelized.lower().replace(separator, " ")
    return cls.ltrim(string.capwords(uncamelized).replace(" ", ""), separator)


# 驼峰转为下划线模式
@classmethod
def uncamelize(cls, camelCaps, separator="_"):
    pattern = re.compile(r'([A-Z]{1})')
    sub = re.sub(pattern, separator + r'\1', camelCaps).lower()
    return sub


# 拆分文本，每行处理,将每行结果合并为数组返回
def split_to_line(str, deal_line_func, *args):
    str = "" if str == None else str
    ret = []
    lines = str.splitlines(keepends=False)
    for line in lines:
        line = line.strip()
        if len(line.strip()) == 0:
            continue
        res = deal_line_func(line, *args)
        if res != None:
            ret.append(res)
    return ret


# 转成一行
def split_to_one_line(str):
    def _lamda(line):
        return line

    list = split_to_line(str, _lamda)
    return "".join(list)


# 匹配中间的字符串
def match_middle_str(str="", front="", behind=""):
    # str = '基金规模：59.29亿元（2019-03-31）'
    # front = '基金规模：'
    # behind = '亿元'
    # re.compile(r'')
    pt = eval("r'" + front + '(.*?)' + behind + "'")
    # print(pt)
    pattern = re.compile(pt)
    # pattern = re.compile(r'基金规模：(.*?)亿元')
    ret = pattern.findall(str)
    # print(ret)
    return ret


# 去掉头尾字符串
def cut_head_tail_str(str, head_str, tail_str):
    str = str.strip()
    if not str.startswith(head_str) or not str.endswith(tail_str):
        return str
    return str[len(head_str):-len(tail_str)]


# 加载jsonp数据
def loads_jsonp(_jsonp):
    try:
        return json.loads(re.match(".*?({.*}).*", _jsonp, re.S).group(1))
    except:
        raise ValueError('Invalid Input')


def get_uuid():
    return str(uuid.uuid4())


# 随机几位数
def rand_int(lengh=3):
    return random.randint(math.pow(10, lengh), math.pow(10, lengh + 1))


def rand_str(n=10):
    ran_str = ''.join(random.sample(string.ascii_letters + string.digits, n))
    return ran_str


# 返回匹配到的IP数组
def match_ip(str):
    result = re.findall(r"\b(?:[0-9]{1,3}\.){3}[0-9]{1,3}\b", str)
    if result:
        return result
    return []


# 过滤得到字符串中的数字
def filter_num(str):
    ret = re.findall("\d+", str)[0]
    # print(ret)
    return ret


def test():
    str = '''date_type时间类型
date_value时间值
group_id技能组
csat满意数
dsat不满意数
feedback_num总反馈数
csat_rate满意率:满意评价数/总留评量
csat_rate_compare_prev满意率环比
dsat_rate不满意率:不满意评价数/总留评量
dsat_rate_compare_prev不满意率环比
pickup_30_num30秒接起数
pickup_num总接起数
server_num服务总数
pickup_30_rate30秒接起率:30秒内对话的对话量/总访问量
pickup_30_compare_prev30秒接起率环比
pickup_rate接起率:对话量/总访问量
pickup_rate_compare_prev接起率环比
ht_total总对话时间
aht平均会话时间:总会话时间/总接起数
aht_compare_prev平均会话时间环比
'''

    res = split_zh_en(str)
    # print(res)
    flag = False
    for it in res:
        if it[0] == 1:
            print(it[1] + " STRING  COMMENT '", end='')
            flag = False
        if it[0] == 2:
            print(it[1] + "'")
            flag = True


if __name__ == '__main__':
    # test()
    # match_middle_str('', '', "")
    str = '2019-08-27 14:35:15,672 INFO  ZPROXY-RESPONSE-FINISH-DIGEST - [a8536c38-cdcd-403e-ae78-33ca0e993236,3e139f3e143cdaed9f47b42d807e8a6c,de833431e1978123729e521026f99cb2,tr,GZ00A,100.88.56.182,UTW,11.166.215.76,SofaRPC,com.alipay.mrchprod.common.service.facade.merchant.MrchProdClassificationMngFacade:1.0@DEFAULT,queryClassification,Y,SUCCESS,imif,mrchprod,1ms,11ms,11ms,5000ms,GEN>ZC]';
    str = '2019-08-27 06:35:15.593,imif,a8536c38-cdcd-403e-ae78-33ca0e993236,0.1.2.1,com.alipay.fc.fcbuservice.common.service.facade.OperatorFacade:1.0,queryByName,TR,SYNC,11.166.190.74:12200,ipaybuservice,UQX,,,,00,1027B,863B,3ms,0ms,0ms,3ms,SofaBizProcessor-4-thread-70,CFS,,F,,11.166.215.76,34604,GZ00A,F,,group4=LINKE_EI61466228_1428&tenantId=ALIPW3SG&abskey=999&group=LINKE_EI61466228_1428&FC_EVENT_CONTEXT={"operator":"jinlong.rhj"%2C"properties":{"BUSERVICE_REGION":"US"%2C"appVersion":""%2C"appName":"ipaycrm"%2C"appId":"1161707040000000101"%2C"tntInstId":"ALIPW3SG"%2C"packageVersion":"126.0002"%2C"deployId":""%2C"appAlias":"ipaycrm"}%2C"tntInstId":"ALIPW3SG"%2C"traceID":"a8536c38-cdcd-403e-ae78-33ca0e993236"}&';
    # print(match_ip(str))
    print(rand_int(2))



from dingtalkchatbot.chatbot import DingtalkChatbot

from pylib.concurrent_util import async_run


@async_run
def send_markdown_async(webhook, title, content,is_at_all=False,at_mobiles=[]):
    # Markdown消息@所有人
    dd = DingtalkChatbot(webhook)
    dd.send_markdown(title, text=content,
                     is_at_all=is_at_all,at_mobiles=at_mobiles)



from urllib.parse import urlparse, quote, unquote


# 获取url问号前面的网关地址
def get_url_gateway(url):
    arr = url.split("?")
    # print(arr[0])
    return arr[0]


# 获取url中的参数对
def get_url_params_map(url):
    arr = url.split("?")
    if len(arr) < 2:
        return {}
    paramstr = arr[1]
    parr = paramstr.split("&")
    # print(parr)
    map = {}
    for str in parr:
        kvarr = str.split("=")
        # print(kvarr)
        if len(kvarr) == 2:
            map[kvarr[0]] = kvarr[1]
    # print(map)
    return map


def get_host(url):
    urlgroup = urlparse(url)
    return urlgroup.scheme + '://' + urlgroup.hostname


def urlencode(str):
    return quote(str, 'utf-8')


def urldecode(str):
    return unquote(str, 'utf-8')


# 将字典转为url参数
def convert_dict_to_url_param(dict={}):
    lst = []
    for (k, v) in dict.items():
        v = str(v)
        lst.append("%s=%s" % (k, urlencode(v)))
    return "&".join(lst)


# 将url参数转为字典
def convert_url_param_to_dict(paramstr=''):
    dict = {}
    arr = paramstr.split('&')
    for item in arr:
        # print(item)
        arr2 = item.split('=')
        if len(arr) > 0:
            dict[arr2[0]] = urldecode(arr2[1])
    return dict


if __name__ == '__main__':
    paramstr = '_input_charset=utf-8&ctoken=6hRNQPoKflJdUxqO&start=0&limit=20&pageIndex=1&resourceType=BLOCK&appName=&orgId=&securityLevel=&optCode=FUNC_CRM_MERCHANT_SITE_QUERY&resInfo=&optCodeDesc=&url='
    print(convert_url_param_to_dict(paramstr))



#!-*-encoding:utf-8-*-

import redis
import json
import types
import collections
from threading import Thread
import time
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor

"""
"""
pubsub_channel_one = 'task:pubsub:channel:one'
pubsub_channel_two = 'task:pubsub:channel:two'
pubsub_channel_three = 'task:pubsub:channel:three'
channel_lst = [pubsub_channel_one, pubsub_channel_two, pubsub_channel_three]


class Task(object):
    def __init__(self):
        self.rcon = redis.StrictRedis(host="localhost", db=5)
        self.ps = self.rcon.pubsub()
        self.ps.subscribe(*channel_lst)

    def listen_task(self):
        for i in self.ps.listen():
            if i['type'] == 'message':
                print("Task get", i['channel'], i['data'])


class PubSubSever(object):
    """
    发布订阅服务器， 使用 非阻塞的方式+ 进程池实现， 为每个channel的handler 开辟一个进程
    """

    def __init__(self, redis_conn_pool, channel_lst=[], max_worker_num=10, pool_type='proc'):
        self.rcon = redis.Redis(connection_pool=redis_conn_pool)
        # 初始化线程池
        self.process_pool = ProcessPoolExecutor(max_worker_num) if pool_type == 'proc' else ThreadPoolExecutor(
            max_worker_num)
        self.channel_lst = channel_lst
        self.registed_handler_dic = collections.defaultdict(list)
        self.pubsub = self.rcon.pubsub()

    def init_sub(self):
        self.pubsub.subscribe(*self.channel_lst)

    def regist_handler(self, channel, handler):

        if channel not in self.registed_handler_dic:
            self.pubsub.subscribe(channel)
            self.channel_lst.append(channel)

        self.registed_handler_dic[channel].append(handler)

    def decode_message(self, message):
        return json.loads(message)

    def parse_message(self, message):
        msg = self.decode_message(message)
        return msg['channel'], msg['data']

    def notify(self, channel, data):
        channel = channel.decode('utf-8')
        idx = 0
        for handler in self.registed_handler_dic[channel]:
            if isinstance(handler, types.FunctionType) or hasattr(handler, '__call__'):
                # print("取出handle")
                # 将handler 和数据封装到一个进程任务中
                self.process_pool.submit(handler, data)

    def event_loop(self):
        """
        服务器以非阻塞的方式运行
        :return:
        """
        while True:
            item = self.pubsub.get_message()  # 非阻塞方式
            # for item in self.pubsub.listen():     # 阻塞方式开启循环
            if not item:
                continue
            if item['type'] == 'message':
                print(item)
                # channel,data = self.parse_message(item)
                channel, data = item['channel'], item['data']
                self.notify(channel, data)

    def close(self):
        self.pubsub.close()
        self.rcon.reset()
        self.process_pool.shutdown()


# =================== handle 各种业务 函数 =========================================
interval = 10


def channel_one_handle_one(data):
    """
    :param data:
    :return:
    """
    print("开启进程handle 1 处理channel 1 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_two_handle_one(data):
    print("开启进程handle 1 处理channel 2 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_three_handle_one(data):
    print("开启进程handle 1 处理channel 3 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_one_handle_two(data):
    """
    :param data:
    :return:
    """
    print("开启进程handle 2 处理channel 1 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_two_handle_two(data):
    print("开启进程handle 2 处理channel 2 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


def channel_three_handle_two(data):
    print("开启进程handle 2 处理channel 3 的信息")
    time.sleep(interval)
    print("返回结果:")
    print(data)
    return


class Channel_one_handler_three(object):
    def __init__(self):
        pass

    def __call__(self, data):
        print("开启进程handle 类 3 处理channel 1 的信息")
        time.sleep(interval)
        print("返回结果:")
        print(data)
        return


class Channel_two_handler_three(object):
    def __init__(self):
        pass

    def __call__(self, data):
        print("开启进程handle 类 3 处理channel 2 的信息")
        time.sleep(interval)
        print("返回结果:")
        print(data)
        return


def run_server():
    # ===================== params ==================================
    host = "localhost"
    port = 6379
    db = 5
    passwd = None

    # 信道
    pubsub_channel_one = 'task:pubsub:channel:one'
    pubsub_channel_two = 'task:pubsub:channel:two'
    pubsub_channel_three = 'task:pubsub:channel:three'

    # =================================================================
    rcon_pool = redis.ConnectionPool(host=host, port=port, db=db, password=passwd)

    mq_server = PubSubSever(rcon_pool, pool_type='th')

    # 为各个信道注册 handler
    mq_server.regist_handler(pubsub_channel_one, channel_one_handle_one)
    mq_server.regist_handler(pubsub_channel_one, channel_one_handle_two)
    mq_server.regist_handler(pubsub_channel_one, Channel_one_handler_three())

    mq_server.regist_handler(pubsub_channel_two, channel_two_handle_one)
    mq_server.regist_handler(pubsub_channel_two, channel_two_handle_two)
    mq_server.regist_handler(pubsub_channel_two, Channel_two_handler_three())

    mq_server.regist_handler(pubsub_channel_three, channel_three_handle_one)
    mq_server.regist_handler(pubsub_channel_three, channel_three_handle_two)

    # 开始订阅各个handler 对应的channel
    mq_server.init_sub()

    try:
        mq_server.event_loop()
    except InterruptedError as e:
        mq_server.close()


if __name__ == '__main__':
    # print('listen task channel')
    # Task().listen_task()
    run_server()



# coding:utf-8
import time
import redis

import json
import types
import collections
from concurrent.futures import ProcessPoolExecutor, ThreadPoolExecutor


class redisClient():
    def __init__(self, **kwargs):
        """
        init topic object
        :param kwargs:
        """
        super().__init__()
        self.host = kwargs.get('host')
        self.port = kwargs.get('port')
        self.db = kwargs.get('db')
        self.password = kwargs.get('password')
        self.pool = redis.ConnectionPool(host=self.host, port=self.port, db=self.db, password=self.password)
        r = redis.Redis(connection_pool=self.pool)
        r.execute_command('config set notify-keyspace-events KEA')  # 发布端，判断如果是第一次就执行

    def pub(self, channel, message):
        rc = redis.StrictRedis(connection_pool=self.pool)
        rc.publish(channel, message)  # 发布消息到liao

    def getRedis(self):
        rc = redis.StrictRedis(connection_pool=self.pool)
        return rc

    # def sub(self):
    #     rc = redis.StrictRedis(connection_pool=self.pool)
    #     ps = rc.pubsub()
    #     ps.subscribe('liao')  # 从liao订阅消息
    #     for item in ps.listen():  # 监听状态：有消息发布了就拿过来
    #         if item['type'] == 'message':
    #             print(item['channel'])
    #
    #             print(item['data'])


    def regeist_handler(self, channel, handler):
        if channel not in self.registed_handler_dic:
            self.channel_lst.append(channel)

        self.registed_handler_dic[channel].append(handler)


class PubSubSever(object):
    """
    发布订阅服务器， 使用 非阻塞的方式+ 进程池实现， 为每个channel的handler 开辟一个进程
    """

    def __init__(self, redis_conn_pool, channel_lst=[], max_worker_num=10, pool_type='thread'):
        self.rcon = redis.StrictRedis(connection_pool=redis_conn_pool)
        # 初始化线程池
        self.process_pool = ProcessPoolExecutor(max_worker_num) if pool_type == 'proc' else ThreadPoolExecutor(
            max_worker_num)
        self.channel_lst = channel_lst
        self.registed_handler_dic = collections.defaultdict(list)
        self.pubsub = self.rcon.pubsub()

    def init_sub(self):
        self.pubsub.subscribe(*self.channel_lst)

    def regist_handler(self, channel, handler):

        if channel not in self.registed_handler_dic:
            print("regist_handler, channel:", channel, ", hander:", handler)
            if '*' in str(channel):
                # 同时订阅多个频道，要用psubscribe
                self.pubsub.psubscribe(channel)
            else:
                self.pubsub.subscribe(channel)
            self.channel_lst.append(channel)

        self.registed_handler_dic[channel].append(handler)

    def decode_message(self, message):
        return json.loads(message)

    def parse_message(self, message):
        msg = self.decode_message(message)
        return msg['channel'], msg['data']

    def notify(self, channel, data):
        channel = channel.decode('utf-8')
        idx = 0
        for handler in self.registed_handler_dic[channel]:
            # print(channel, handler)
            if isinstance(handler, types.FunctionType) or hasattr(handler, '__call__'):
                # print("取出handle")
                # 将handler 和数据封装到一个进程任务中
                self.process_pool.submit(handler, data)

    def event_loop(self):
        """
        服务器以非阻塞的方式运行
        :return:
        """
        while True:
            item = self.pubsub.get_message()  # 非阻塞方式
            # for item in self.pubsub.listen():     # 阻塞方式开启循环
            # print("get_message item: ", item)
            time.sleep(1)
            if not item:
                continue
            if item['type'] == 'message':
                print("event_loop: ", item)
                # channel,data = self.parse_message(item)
                channel, data = item['channel'], item['data']
                self.notify(channel, data)
                # time.sleep(1)
            if item['type'] == 'pmessage':
                pattern, channel, data = item['pattern'], item['channel'], item['data']
                # 如果是过期消息，就将key返回个handler处理
                if data == bytes('expired', encoding="utf8"):
                    key = str(channel).split(":")[1]
                    self.notify(pattern, key)

    def close(self):
        self.pubsub.close()
        self.rcon.reset()
        self.process_pool.shutdown()


if __name__ == '__main__':
    redisClient = redisClient(host="t.navyran.top", port=6399, db=1, password="Jinlong1234")
    # redisClient.sub()
    # redisClient.pub()

    # p.unsubscribe('spub')
    redisClient.pub("channel1", "haha")



# encoding:utf8
import logging
import sys
from loguru import logger as log

# 创建名为'spam_application'的记录器
logger = logging.getLogger('myapp')
logger.setLevel(logging.INFO)

log.add('./python.log')
loginfo = log.info

# 控制台输出

sh = logging.StreamHandler(sys.stdout)
sh.setLevel(logging.INFO)

# 创建级别为DEBUG的日志处理器
fh = logging.FileHandler('./python.log')
fh.setLevel(logging.INFO)

# 创建级别为ERROR的控制台日志处理器
ch = logging.StreamHandler()
ch.setLevel(logging.ERROR)

# 创建格式器，加到日志处理器中
formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
fh.setFormatter(formatter)
ch.setFormatter(formatter)

logger.addHandler(sh)
logger.addHandler(fh)
logger.addHandler(ch)


def getLogger(name, file):
    logger.name = name
    logger.addHandler(logging.FileHandler(file))
    return logger



import yagmail
from .concurrent_util import async_run


def send_mail(receivers, subject="", content="", files=[]):
    # 链接邮箱服务器
    yag = yagmail.SMTP(user="kbtest@aliyun.com", password="Kbtestjinlong1234", host='smtp.aliyun.com')
    # 邮箱正文
    contents = ['This is the body, and here is just text http://somedomain/image.png',
                'You can find an audio file attached.', '/local/path/song.mp3']
    # 发送邮件
    yag.send(receivers, subject, content, attachments=files)


@async_run
def send_mail_async(receivers, subject="", content="", files=[]):
    send_mail(receivers, subject, content, files)


if __name__ == '__main__':
    send_mail('ranhaijun1129@163.com', "a", 'b', files=['/Users/jinlong.rhj/Downloads/gotty_darwin_amd64.tar.gz'])



import requests
import threading
from multiprocessing import cpu_count
import progressbar


class downloader:
    def __init__(self, url, filepath, header=None, proxy=None):
        self.url = url
        self.num = cpu_count() * 2
        self.name = filepath
        self.header = header
        self.proxy = proxy
        r = requests.head(self.url)
        # 获取文件大小
        self.total = int(r.headers['Content-Length'])
        # print(self.total)

    # 获取每个线程下载的区间
    def get_range(self):
        ranges = []
        offset = int(self.total / self.num)
        for i in range(self.num):
            if i == self.num - 1:
                ranges.append((i * offset, ''))
            else:
                ranges.append((i * offset, (i + 1) * offset))
        return ranges  # [(0,100),(100,200),(200,"")]

    # 通过传入开始和结束位置来下载文件
    def download(self, start, end, n):
        headers = {'Range': 'Bytes=%s-%s' % (start, end), 'Accept-Encoding': '*'}
        res = requests.get(self.url, headers=headers)
        print("%s-%s download success, part %s" % (start, end, n))
        # 将文件指针移动到传入区间开始的位置
        self.fd.seek(start)
        self.fd.write(res.content)

    def multi_thread_download(self):
        self.fd = open(self.name, "wb")

        thread_list = []
        n = 0

        for ran in self.get_range():
            # 获取每个线程下载的数据块
            start, end = ran
            n += 1
            print("start part: %s" % n)
            thread = threading.Thread(target=self.download, args=(start, end, n))
            thread.start()
            thread_list.append(thread)

        for i in thread_list:
            # 设置等待，避免上一个数据块还没写入，下一数据块对文件seek，会报错
            i.join()

        self.fd.close()

    def download_progressbar(self):
        # requests.packages.urllib3.disable_warnings()

        response = requests.request("GET", self.url, stream=True, data=None, headers=self.header, proxies=self.proxy)

        total_length = int(response.headers.get("Content-Length"))
        print("total_length", total_length)
        with open(self.name, 'wb') as f:
            # widgets = ['Processed: ', progressbar.Counter(), ' lines (', progressbar.Timer(), ')']
            # pbar = progressbar.ProgressBar(widgets=widgets)
            # for chunk in pbar((i for i in response.iter_content(chunk_size=1))):
            #     if chunk:
            #         f.write(chunk)
            #         f.flush()

            widgets = ['Progress: ', progressbar.Percentage(), ' ',
                       progressbar.Bar(marker='#', left='[', right=']'), ' ', progressbar.Timer(),
                       ' ', progressbar.ETA(), ' ', progressbar.FileTransferSpeed()]
            pbar = progressbar.ProgressBar(widgets=widgets, maxval=total_length).start()
            for chunk in response.iter_content(chunk_size=1000):

                if chunk:
                    f.write(chunk)
                    f.flush()
                # print(len(chunk))
                pbar.update(len(chunk) + 1)
            pbar.finish()


if __name__ == "__main__":
    downloader("https://jaist.dl.sourceforge.net/project/liteide/x36/liteidex36.macos-qt5.9.5.zip",
               "liteide.zip").multi_thread_download()



import codecs
import time, os, sys
import shutil
import glob

'''

# os 模块

os.sep 可以取代操作系统特定的路径分隔符。windows下为 '\\'
os.name 字符串指示你正在使用的平台。比如对于Windows，它是'nt'，而对于Linux/Unix用户，它是 'posix'
os.getcwd() 函数得到当前工作目录，即当前Python脚本工作的目录路径
os.getenv() 获取一个环境变量，如果没有返回none
os.putenv(key, value) 设置一个环境变量值
os.listdir(path) 返回指定目录下的所有文件和目录名
os.remove(path) 函数用来删除一个文件
os.system(command) 函数用来运行shell命令
os.linesep 字符串给出当前平台使用的行终止符。例如，Windows使用 '\r\n'，Linux使用 '\n' 而Mac使用 '\r'
os.path.split(path)  函数返回一个路径的目录名和文件名
os.path.isfile() 和os.path.isdir()函数分别检验给出的路径是一个文件还是目录
os.path.exists() 函数用来检验给出的路径是否真地存在
os.curdir  返回当前目录 ('.')
os.mkdir(path) 创建一个目录
os.makedirs(path) 递归的创建目录
os.chdir(dirname) 改变工作目录到dirname    
os.path.getsize(name) 获得文件大小，如果name是目录返回0L
os.path.abspath(name) 获得绝对路径
os.path.normpath(path) 规范path字符串形式
os.path.splitext()  分离文件名与扩展名
os.path.join(path,name) 连接目录与文件名或目录
os.path.basename(path) 返回文件名
os.path.dirname(path) 返回文件路径
os.walk(top,topdown=True,onerror=None)  遍历迭代目录
os.rename(src, dst)  重命名file或者directory src到dst 如果dst是一个存在的directory, 将抛出OSError. 在Unix, 如果dst在存且是一个file, 如果用户有权限的话，它将被安静的替换. 操作将会失败在某些Unix 中如果src和dst在不同的文件系统中. 如果成功, 这命名操作将会是一个原子操作 (这是POSIX 需要). 在 Windows上, 如果dst已经存在, 将抛出OSError，即使它是一个文件. 在unix，Windows中有效。
os.renames(old, new) 递归重命名文件夹或者文件。像rename()
# shutil 模块

shutil.copyfile( src, dst) 从源src复制到dst中去。当然前提是目标地址是具备可写权限。抛出的异常信息为IOException. 如果当前的dst已存在的话就会被覆盖掉
shutil.move( src, dst)  移动文件或重命名
shutil.copymode( src, dst) 只是会复制其权限其他的东西是不会被复制的
shutil.copystat( src, dst) 复制权限、最后访问时间、最后修改时间
shutil.copy( src, dst)  复制一个文件到一个文件或一个目录
shutil.copy2( src, dst)  在copy上的基础上再复制文件最后访问时间与修改时间也复制过来了，类似于cp –p的东西
shutil.copy2( src, dst)  如果两个位置的文件系统是一样的话相当于是rename操作，只是改名；如果是不在相同的文件系统的话就是做move操作
shutil.copytree( olddir, newdir, True/Flase)
把olddir拷贝一份newdir，如果第3个参数是True，则复制目录时将保持文件夹下的符号连接，如果第3个参数是False，则将在复制的目录下生成物理副本来替代符号连接
shutil.rmtree( src ) 递归删除一个目录以及目录内的所有内容

'''


# 递归删除文件夹
def removerf(src):
    shutil.rmtree(src)


def mkdir(path):
    # 引入模块
    import os

    # 去除首位空格
    path = path.strip()
    # 去除尾部 \ 符号
    path = path.rstrip("\\")

    # 判断路径是否存在
    # 存在     True
    # 不存在   False
    isExists = os.path.exists(path)

    # 判断结果
    if not isExists:
        # 如果不存在则创建目录
        print(path + ' 创建成功')
        # 创建目录操作函数
        os.makedirs(path)
        return True
    else:
        # 如果目录存在则不创建，并提示目录已存在
        print(path + ' 目录已存在')

        return False


def getCurrentDirAbsPath():
    abspath = os.path.dirname(os.path.abspath(sys.argv[0]))
    return abspath


#########start 获取文件路径、文件名、后缀名############
def get_filePath_fileName_fileExt(filename):
    (filepath, tempfilename) = os.path.split(filename);
    (shotname, extension) = os.path.splitext(tempfilename);
    return filepath, shotname, extension


# 获取所有文件路径列表
def getAllFilesInDir(basedir):
    L = []
    for root, dirs, files in os.walk(basedir):
        for file in files:
            L.append(os.path.join(root, file))
    return L


# 获取所有子目录路径列表
def getAllDirsInDir(basedir):
    L = []
    for root, dirs, files in os.walk(basedir):
        for dir in dirs:
            L.append(os.path.join(root, dir))
    return L


# 获取文件夹大小
def getdirsize(dir):
    size = 0
    for root, dirs, files in os.walk(dir):
        size += sum([os.path.getsize(os.path.join(root, name)) for name in files])
    return size


# 根据后缀名获取所有文件路径列表
def getAllFilesInDirByExt(basedir, ext):
    L = []
    for root, dirs, files in os.walk(basedir):
        for file in files:
            if os.path.splitext(file)[1] == ext:
                L.append(os.path.join(root, file))
    return L


# 删除目录中直接的文件
def removeFileInFirstDir(targetDir):
    for file in os.listdir(targetDir):
        targetFile = os.path.join(targetDir, file)
        if os.path.isfile(targetFile):
            os.remove(targetFile)


# 目录拷贝
def copyDir(sourceDir, targetDir):
    if sourceDir.find(".svn") > 0:
        return
    for file in os.listdir(sourceDir):
        sourceFile = os.path.join(sourceDir, file)
        targetFile = os.path.join(targetDir, file)
        if os.path.isfile(sourceFile):
            if not os.path.exists(targetDir):
                os.makedirs(targetDir)
            if not os.path.exists(targetFile) or (
                        os.path.exists(targetFile) and (os.path.getsize(targetFile) != os.path.getsize(sourceFile))):
                open(targetFile, "wb").write(open(sourceFile, "rb").read())
        if os.path.isdir(sourceFile):
            First_Directory = False
            copyDir(sourceFile, targetFile)


def remove_empty_files_dirs(path):
    """
    CLean empty files, 清理空文件夹和空文件
    :param path: 文件路径，检查此文件路径下的子文件
    :return: None
    """
    files = os.listdir(path)  # 获取路径下的子文件(夹)列表
    for file in files:
        print('Traversal at', file)
        if os.path.isdir(file):  # 如果是文件夹
            if not os.listdir(file):  # 如果子文件为空
                os.rmdir(file)  # 删除这个空文件夹
        elif os.path.isfile(file):  # 如果是文件
            if os.path.getsize(file) == 0:  # 文件大小为0
                os.remove(file)  # 删除这个文件
    print(path, 'Dispose over!')


# 读取文件内容
def read_text(filepath):
    file_object = open(filepath)
    try:
        all_the_text = file_object.read()
    finally:
        file_object.close()
    return all_the_text


# 读取文件内容，建议用这个
def read_content(filepath, charset='utf-8'):
    file_object = codecs.open(filepath, "r", charset)
    try:
        all_the_text = file_object.read()
    finally:
        file_object.close()
    return all_the_text


# 读取文本文件所有行
def read_all_lines(filepath, charset='utf-8'):
    file_object = codecs.open(filepath, "r", charset)
    try:
        list_of_all_the_lines = file_object.readlines()
    finally:
        file_object.close()
    return list_of_all_the_lines


# 限制每次读行大小，callback中传的读出的行
def read_lines_limit_size(filepath, size_kb, line_func):
    file = open(filepath, 'r')
    sizehint = size_kb * 1024  # 200M
    position = 0
    lines = file.readlines(sizehint)
    while not file.tell() - position < 0:
        line_func(lines)
        # 继续读下一批
        position = file.tell()
        lines = file.readlines(sizehint)


# 写文件
def write_file(filepath, content):
    fw = codecs.open(filepath, "w", "utf-8")
    # html = unicode(html, "utf-8")
    fw.write(content)
    fw.close()


# 写入每行组成的列表
def write_file_txt_list(filepath, list):
    fw = codecs.open(filepath, 'w')
    fw.writelines(list)
    fw.close()


# 追加文件
def append_file(filepath, content, encoding='utf-8'):
    if not os.path.exists(filepath):
        fw = codecs.open(filepath, 'w', encoding)
        fw.close()
    fw = codecs.open(filepath, 'a', encoding)
    fw.write(content)
    fw.write(os.linesep)
    fw.close()


# 创建文件，如果不存在才创建
def create_file(filepath, encoding='utf-8'):
    if not os.path.exists(filepath):
        fw = codecs.open(filepath, 'w', encoding)
        fw.close()


'''把时间戳转化为时间: 1479264792 to 2016-11-16 10:53:12'''


def TimeStampToTime(timestamp):
    timeStruct = time.localtime(timestamp)
    return time.strftime('%Y-%m-%d %H:%M:%S', timeStruct)


'''获取文件的大小,结果保留两位小数，单位为MB'''


def get_FileSize(filePath):
    # filePath = unicode(filePath, 'utf8')
    fsize = os.path.getsize(filePath)
    fsize = fsize / float(1024 * 1024)
    return round(fsize, 2)


'''获取文件的访问时间'''


def get_FileAccessTime(filePath):
    # filePath = unicode(filePath, 'utf8')
    t = os.path.getatime(filePath)
    return TimeStampToTime(t)


'''获取文件的创建时间'''


def get_FileCreateTime(filePath):
    # filePath = unicode(filePath, 'utf8')
    t = os.path.getctime(filePath)
    return TimeStampToTime(t)


'''获取文件的修改时间'''


def get_FileModifyTime(filePath):
    # filePath = unicode(filePath, 'utf8')
    t = os.path.getmtime(filePath)
    return TimeStampToTime(t)


# 用于判断文件后缀
# allowed_extensions=set(['txt','png','jpg','xls','JPG','PNG','xlsx','gif','GIF'])
def allowed_file(filename, allowed_extensions):
    return '.' in filename and filename.rsplit('.', 1)[1].lower() in allowed_extensions


# 支持正则表达式 '/Volumes/Public/5演讲/xs金融(18.12.10完结)/6月/*/*.mp3'
def find_files(path_rule, recursive=False):
    files1 = glob.glob(path_rule, recursive=recursive)
    return files1


# 检查文本是否在文件中
def is_in_file(filePath, match_content):
    if not os.path.exists(filePath):
        return False
    content = read_content(filePath)
    if content and content.find(match_content) > -1:
        return True
    return False


def is_exsit(file):
    return os.path.exists(file)



from logging import exception


class Queue():
    def __init__(self, size):
        self.size = size
        self.front = -1
        self.rear = -1
        self.queue = []

    def enqueue(self, ele):  # 入队操作
        if self.isfull():
            raise exception("queue is full")
        else:
            self.queue.append(ele)
            self.rear = self.rear + 1

    def dequeue(self):  # 出队操作
        if self.isempty():
            raise exception("queue is empty")
        else:
            self.front = self.front + 1
            return self.queue[self.front]

    def isfull(self):
        return self.rear - self.front + 1 == self.size

    def isempty(self):
        return self.front == self.rear



from flask import Flask, render_template, request, jsonify, Blueprint


def getRequestParam(paramKey):
    v = request.form.get(paramKey)
    if v == None:
        v = request.args.get(paramKey)
    return v



import sys, os

curPath = os.path.abspath(os.path.dirname(__file__))
# 加载上几级
for i in range(2):
    curPath = os.path.split(curPath)[0]
    sys.path.append(curPath)

from loguru import logger
from pylib import *
from mylib import *

from string import Template
import json



import requests


def download_mini(url, file):
    r = requests.get(url)  # create HTTP response object
    with open(file, 'wb') as f:
        f.write(r.content)


def download(url, file):
    r = requests.get(url, stream=True)
    with open(file, "wb") as f:
        for chunk in r.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)


if __name__ == '__main__':
    image_url = "https://www.python.org/static/community_logos/python-logo-master-v3-TM.png"
    # download_mini(image_url, "python_logo.png")
    download(image_url, "python_logo.png")



from concurrent.futures import ThreadPoolExecutor, ProcessPoolExecutor, wait, as_completed


# 不要求返回结果
def threadPoolExecutor(pool_size, task_func, params):
    pool = ThreadPoolExecutor(max_workers=pool_size)  # 创建一个最大可容纳2个task的线程池

    for i in range(pool_size):
        # print(task_func)
        if params:
            future = pool.submit(task_func, params)  # 往线程池里面加入一个task
        else:
            future = pool.submit(task_func)

            # print(future.done())  # 判断task1是否结束
            # time.sleep(3)


            # print(future1.result())  # 查看task1返回的结果


# 不要求返回结果
def processPoolExecutor(pool_size, task_func, params):
    pool = ProcessPoolExecutor(max_workers=pool_size)  # 创建一个最大可容纳2个task的线程池

    for i in range(pool_size):
        # print(task_func)
        if params:
            future = pool.submit(task_func, params)  # 往线程池里面加入一个task
        else:
            future = pool.submit(task_func)


# 等待线程池执行完成，返回futures
def waitThreadPoolExecute(pool_size, task_func, params):
    pool = ThreadPoolExecutor(pool_size)
    futures = []
    for x in range(pool_size):
        future = pool.submit(task_func, params)
        futures.append(future)

    return wait(futures)



import threading, logging
import multiprocessing
from multiprocessing import cpu_count
from concurrent import futures
from concurrent.futures import ThreadPoolExecutor, wait
from . import date_util


def getDefaultThreadPool(workers=cpu_count() * 2):
    if workers <= 0:
        workers = cpu_count() * 2
    return ThreadPoolExecutor(max_workers=workers)


def getDefaultProcessPoll(workers=cpu_count()):
    if workers <= 0:
        workers = cpu_count()
    pool = multiprocessing.Pool(workers)
    return pool


# 指定执行次数,没有参数，不关心结果
def threadPoolExecute(task_func, run_times=0, workers=0):
    pool = getDefaultThreadPool(workers)
    for i in range(0, run_times):
        pool.submit(task_func)  # 往线程池里面加入一个task


# 带参数列表异步执行,不阻塞主线程
# task_func函数处理单条，接收params中的单个参数
def threadPoolExecuteWithParams(task_func, params=[], workers=0):
    pool = getDefaultThreadPool(workers)
    for param in params:
        pool.submit(task_func, param)


# 同步返回与提交参数顺序一致的结果迭代器，调用方自己遍历处理
def threadPoolExecuteReturn(task_func, params=[], workers=0):
    pool = getDefaultThreadPool(workers)
    res_iter = pool.map(task_func, params)
    # for res in res_iter:
    #     print("ret: ", res)
    return res_iter


# 阻塞模式，等待调用完成，返回无序map
def threadPoolExecuteSyncReturn(task_func, params=[], timeout=None, workers=0):
    pool = getDefaultThreadPool(workers)

    mp = dict()
    ret = dict()

    for param in params:
        future = pool.submit(task_func, param)
        mp[future] = param

    for future in futures.as_completed(mp):
        param = mp[future]
        try:
            data = future.result()
            ret[param] = data
        except Exception as exc:
            logging.exception(exc)
            print(exc, ',param:', param)
        else:
            # print('param:', param)
            None

    return ret


# 调用方阻塞，等待处理,params是需要遍历的参数列表,如果函数需要多个参数， 需要自行封装成一个参数
def threadPoolExecuteSync(task_func, params=[], timeout=None, workers=0, show_progress=False):
    pool = getDefaultThreadPool(workers)

    i = 0
    future_tasks = [pool.submit(task_func, param) for param in params]
    try:
        result = wait(future_tasks, timeout=timeout, return_when=futures.ALL_COMPLETED)

        done_set = result[0]
        undone_set = result[1]
        for future in done_set:
            i = i + 1
            if show_progress:
                print('progress: %s/%s' % (i, len(params)))
            data = future.result()
            # print('第一个网页任务完成 url:%s , len:%d bytes! ' % (resp.url, len(resp.text)))
    except Exception as e:
        print('exception :', e)
        logging.exception(e)
    finally:
        pass


# 非阻塞，异步调用，回调模式，future会传给回调函数
def threadPoolExecuteWithCallback(task_func, params=[], callback_func=None, workers=0):
    pool = getDefaultThreadPool(workers)
    mp = dict()
    ret = dict()
    for param in params:
        future = pool.submit(task_func, param)
        mp[future] = param
        future.add_done_callback(callback_func)

        # for future in futures.as_completed(mp):
        #     param = mp[future]
        #     try:
        #         data = future.result()
        #         ret[param] = data
        #     except Exception as exc:
        #         print(exc, ',param:', param)
        #     else:
        #         # print('param:', param)
        #         None
        #
        # return ret


def processPoolExecuteAsync(task_func, params=[], workers=0):
    pool = getDefaultProcessPoll(workers)

    for param in params:
        pool.apply_async(task_func, (param))  # 维持执行的进程总数为processes，当一个进程执行完毕后会添加新的进程进去


def processPoolExecuteSync(task_func, params=[], workers=0):
    pool = getDefaultProcessPoll(workers)

    for param in params:
        pool.apply(task_func, (param,))  # 维持执行的进程总数为processes，当一个进程执行完毕后会添加新的进程进去

    pool.close()  # 关闭进程池，表示不能在往进程池中添加进程
    pool.join()  # 等待进程池中的所有进程执行完毕，必须在close()之后调用


# 获取当前线程id
def getCurrentThreadId():
    return threading.currentThread().ident


# 异步执行注解
def async_run(f):
    def wrapper(*args, **kwargs):
        thr = threading.Thread(target=f, args=args, kwargs=kwargs)
        thr.start()

    return wrapper


# 本地并发锁
lock = {}


# 超时锁，判断是否超时，key用于区分不同业务，timeout单位是秒
def lock_timeout(key, timeout=60):
    global lock
    lockdate = lock.get(key, None)
    if not lockdate:
        lock[key] = date_util.getTimestamps() - timeout

    if (date_util.getTimestamps() - lock[key]) > timeout:
        lock[key] = date_util.getTimestamps()
        return True
    return False


counter = {}


# 计数器初始化
def counter_init(key):
    global counter
    counter[key] = 0


# 计数器累加
def counter_accumulate(key, print=False):
    global counter
    curr = counter.get(key, 0)
    counter[key] = curr + 1
    if print:
        info = 'counter key: %s , current: %d' % (key, counter[key])
        logging.info(info)
    return counter[key]


def func(msg):
    print(multiprocessing.current_process().name + '-' + msg)


if __name__ == '__main__':
    processPoolExecuteSync(func, range(1, 10))



# -*- encoding:utf8 -*-
'''
@author: crazyant.net
@version: 2013-10-22

封装的mysql常用函数
'''

import MySQLdb


class DB():
    def __init__(self, DB_HOST, DB_PORT, DB_USER, DB_PWD, DB_NAME):
        self.DB_HOST = DB_HOST
        self.DB_PORT = DB_PORT
        self.DB_USER = DB_USER
        self.DB_PWD = DB_PWD
        self.DB_NAME = DB_NAME

        self.conn = self.getConnection()

    def getConnection(self):
        return MySQLdb.Connect(
            host=self.DB_HOST,  # 设置MYSQL地址
            port=self.DB_PORT,  # 设置端口号
            user=self.DB_USER,  # 设置用户名
            passwd=self.DB_PWD,  # 设置密码
            db=self.DB_NAME,  # 数据库名
            charset='utf8'  # 设置编码
        )

    def query(self, sqlString):
        cursor = self.conn.cursor()
        cursor.execute(sqlString)
        returnData = cursor.fetchall()
        cursor.close()
        self.conn.close()
        return returnData

    def update(self, sqlString):
        cursor = self.conn.cursor()
        cursor.execute(sqlString)
        self.conn.commit()
        cursor.close()
        self.conn.close()


if __name__ == "__main__":
    db = DB('127.0.0.1', 3306, 'root', '', 'wordpress')
    print(db.query("show tables;"))



import os
from pyftpdlib.authorizers import DummyAuthorizer
from pyftpdlib.handlers import FTPHandler
from pyftpdlib.servers import FTPServer
from ftplib import FTP


 # '''
# #用户名     密码       权限         目录
## root      12345      elradfmwM    /home/
# huangxm     12345      elradfmwM    /home/
# test       12345      elradfmwM    /home/huangxm
# '''
def get_user(userfile):
    #定义一个用户列表
    user_list = []
    with open(userfile) as f:
        for line in f:
            if not line.startswith('#') and line:
                if len(line.split()) == 4:
                    user_list.append(line.split())
                else:
                    print("user.conf配置错误")
    return user_list

def ftp_server():
    # 实例化用户授权管理
    authorizer = DummyAuthorizer()
    # authorizer.add_user('user', '12345', dir_path, perm='elradfmwMT')  # 添加用户 参数:username,password,允许的路径,权限

    # user_list = get_user('conf/user.py')
    # for user in user_list:
    #     name, passwd, permit, homedir = user
    #     try:
    #         authorizer.add_user(name, passwd, homedir, perm=permit)
    #     except Exception as e:
    #         print(e)

    authorizer.add_anonymous(os.getcwd())  # 这里是允许匿名用户,如果不允许删掉此行即可

    # 实例化FTPHandler
    handler = FTPHandler
    handler.authorizer = authorizer

    # 设定一个客户端链接时的标语
    handler.banner = "pyftpdlib based ftpd ready."


    # handler.masquerade_address = '151.25.42.11'#指定伪装ip地址
    handler.passive_ports = range(60000, 65535)#指定允许的端口范围

    address = ("localhost", 2121)  # FTP一般使用21,20端口
    server = FTPServer(address, handler)  # FTP服务器实例

    # set a limit for connections
    server.max_cons = 256
    server.max_cons_per_ip = 5

    # 开启服务器
    server.serve_forever()


def ftp_client():
    ftp = FTP()
    ftp.connect('localhost', 21)  # localhost改成服务器ip地址
    ftp.login(user='user', passwd='12345')

    file = open('f://ftpdownload/test.txt', 'wb')
    ftp.retrbinary("RETR test.txt", file.write, 1024)  # 从服务器上下载文件 1024字节一个块
    ftp.set_debuglevel(0)
    ftp.close()

if __name__ == '__main__':
    ftp_server()


#!/usr/bin/python
# coding=utf-8
import paramiko, time


def ssh_connect(_host, _username, _password):
    _ssh_fd = paramiko.SSHClient()
    _ssh_fd.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    try:
        _ssh_fd.connect(_host, username=_username, password=_password)
    except Exception as e:
        print('ssh %s@%s: %s' % (_username, _host, e))
        # exit()
    return _ssh_fd


def ssh_exec_cmd(_ssh_fd, _cmd, timeout=10, get_pty=True):
    return _ssh_fd.exec_command(_cmd, timeout, get_pty)


def ssh_close(_ssh_fd):
    _ssh_fd.close()


def remote_exec(hostname, username, password, cmd, timeout=10, get_pty=True):
    sshd = ssh_connect(hostname, username, password)
    try:
        stdin, stdout, stderr = ssh_exec_cmd(sshd, cmd, timeout, get_pty)
        stderr._set_mode('b')
        err_list = stderr.readlines()
        if len(err_list) > 0:
            print('ERROR:', err_list[0])
            # return ""

        str = ""

        for item in stdout.readlines():
            str = str + item
        ssh_close(sshd)
        return str
    except Exception as e:
        print('ERROR: ssh %s@%s: %s' % (username, hostname, e))


def paramiko_interact(hostname, username, password, cmd, do_with_rst):
    trans = paramiko.Transport((hostname, 22))  # 【坑1】 如果你使用 paramiko.SSHClient() cd后会回到连接的初始状态
    trans.start_client()
    # 用户名密码方式
    trans.auth_password(username, password)
    # 打开一个通道
    channel = trans.open_session()
    channel.settimeout(7200)
    # 获取一个终端
    channel.get_pty()
    # 激活器
    channel.invoke_shell()
    # cmd = 'cd /home/shell_study\r'
    # 发送要执行的命令
    # channel.send(cmd)
    # cmd = 'bash ./study_shell.sh\r' # 【坑2】 如果你使用 sh ./study_shell.sh\r 可能会出现 [: 11: y: unexpected operator 错误
    channel.send(cmd + '\n')
    # 回显很长的命令可能执行较久，通过循环分批次取回回显
    while True:
        time.sleep(0.2)
        rst = channel.recv(1024)
        try:
            rst = rst.decode('utf-8')
        except:
            rst = rst.decode('gbk')
        print(rst)
        # 通过命令执行提示符来判断命令是否执行完成
        # 【坑3】 如果你使用绝对路径，则会在home路径建立文件夹导致与预期不符
        code = do_with_rst(channel, rst)
        code = 0 if code == None else code
        if code < 0:
            break


    channel.close()
    trans.close()
    # channel.invoke_shell()


def scp_upload(hostname, username, password, localpath, remotepath, port=22):
    t = paramiko.Transport((hostname, port))
    t.connect(username=username, password=password)
    sftp = paramiko.SFTPClient.from_transport(t)
    # remotepath='/csdp/user_launcher-1.0-dev/user-1.0-release.jar'
    # localpath= r'D:\workspace\csdp201512041\csdp-ningxia\csdp_user\user\target\user-1.0-release.jar'
    sftp.put(localpath, remotepath)
    t.close()
    print("：） 成功上传 %s 文件。" % remotepath)


def scp_download(hostname, username, password, remotepath, localpath, port=22):
    t = paramiko.Transport((hostname, port))
    t.connect(username=username, password=password)
    sftp = paramiko.SFTPClient.from_transport(t)
    # remotepath='/csdp/user_launcher-1.0-dev/user-1.0-release.jar'
    # localpath= r'D:\workspace\csdp201512041\csdp-ningxia\csdp_user\user\target\user-1.0-release.jar'
    sftp.get(remotepath, localpath)
    t.close()
    print("：） 成功下载 %s 文件。" % localpath)


def main():
    hostname = 'icicada-zth-2.gz00b.stable.alipay.net'
    port = 22
    username = 'log'
    password = 'log_xixihaha'
    cmd = "grep --color=auto 89ef287b-5a56-4d8f-bded-8d6c4c9469db /home/admin/logs/tracelog/rpc-client-digest.log"

    ret = remote_exec(hostname, username, password, cmd)
    print(ret)


if __name__ == "__main__":
    main()



import os

from croniter import croniter
from datetime import datetime
from apscheduler.schedulers.blocking import BlockingScheduler

sched = BlockingScheduler()


def interval(func, seconds):
    sched.add_job(func, 'interval', seconds=seconds)
    sched.start()


# def cron(func):
#     sched.add_job(func, 'cron', month='6-8,11-12', day='3rd fri', hour='0-3')
#     sched.start()
# 返回cron表达式的下次执行时间，表达式规则：分时日月周秒
def next_process_time(cron_str):
    iter = croniter(cron_str, datetime.now())

    iter.get_next(datetime)
    # print(iter.get_next(datetime))
    # print(iter.get_current(datetime))
    # print(type(iter.get_current(datetime)))
    # 打印结果为：2019-02-22 08:00:00
    return iter.get_current(datetime)


# 到下次执行时间的剩余秒数
def next_remaining_seconds(cron_str):
    datetime = next_process_time(cron_str)
    now = datetime.now()
    diff = datetime - now
    return diff.seconds


# 下次执行时间，小于约定时间，则触发
def is_trigger_now(cron_str, diff_seconds=60):
    diff = next_remaining_seconds(cron_str)
    if abs(diff) < diff_seconds:
        return True
    return False


if __name__ == '__main__':
    # next_time("0 8 * * *")
    print(next_process_time("1 * * * *"))
    print(next_remaining_seconds("1 10/12 * * *"))
    print(is_trigger_now("1 10/12 * * *"))



# coding=utf-8
'''
Created on 2013年11月22日

@author: crazyant.net
'''
import shlex, os
import datetime
import subprocess
import time
import sh


def execute_command(cmdstring, cwd=None, timeout=None, shell=False):
    """执行一个SHELL命令
            封装了subprocess的Popen方法, 支持超时判断，支持读取stdout和stderr
           参数:
        cwd: 运行命令时更改路径，如果被设定，子进程会直接先更改当前路径到cwd
        timeout: 超时时间，秒，支持小数，精度0.1秒
        shell: 是否通过shell运行
    Returns: return_code
    Raises:  Exception: 执行超时
    """
    if shell:
        cmdstring_list = cmdstring
    else:
        cmdstring_list = shlex.split(cmdstring)

    sub = subprocess_popen_param_array(cmdstring_list, cwd, shell, timeout)

    return (str(sub.returncode), str(sub.stdout), str(sub.stderr))


# 传数组参数，执行返回
def subprocess_popen_param_array(cmdstring_list, cwd=None, shell=False, timeout=None):
    # 没有指定标准输出和错误输出的管道，因此会打印到屏幕上；
    sub = subprocess.Popen(cmdstring_list, cwd=cwd, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
                           stderr=subprocess.STDOUT, shell=shell,
                           bufsize=4096)
    # subprocess.poll()方法：检查子进程是否结束了，如果结束了，设定并返回码，放在subprocess.returncode变量中

    end_time = datetime.datetime.now() + datetime.timedelta(days=1)

    if timeout:
        end_time = datetime.datetime.now() + datetime.timedelta(seconds=timeout)

    while sub.poll() is None:
        time.sleep(0.1)
        if timeout:
            if end_time <= datetime.datetime.now():
                raise Exception("Timeout：%s" % ' '.join(cmdstring_list))
    # for i in iter(sub.stdout.readline, 'b'):
    #     print(i)
    return sub


def exec_shell(shellstr, charset='utf8'):
    # curl = "curl -H 'Host: zt.wps.cn' -H 'Content-Type: application/json' -H 'Accept: */*' -H 'sid: V02SffcFNB9nA58U3Yx2U_KpyN0dHS400a8c765300001ad274' -H 'User-Agent: Mozilla/5.0 (iPhone; CPU iPhone OS 12_1 like Mac OS X) AppleWebKit/605.1.15 (KHTML, like Gecko) Mobile/16B92 MicroMessenger/6.7.3(0x16070321) NetType/WIFI Language/zh_CN' -H 'Referer: https://servicewechat.com/wx2f333d84a103825d/42/page-frame.html' -H 'Accept-Language: zh-cn' --compressed 'https://zt.wps.cn/2018/clock_in/api/get_question?member=wps'"
    sub = subprocess.Popen(shellstr, cwd=None, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
                           shell=True,
                           bufsize=4096)
    sub.wait()
    # for i in iter(sub.stdout.readline,'b'):
    #     print(i)
    bstr = sub.stdout.read()
    if bstr:
        return bstr.decode(charset)
    return ''
    # print(sub.stdout.read().decode('utf-8'))
    # print(sub.stderr.readlines())


def test2():
    data = ['www.baidu.com',
            'www.csdn.cn']

    for item in data:
        tmpres = os.popen('curl %s' % item).readlines()
        print(tmpres)


def sh_test():
    print(sh.ifconfig("eth0"))
    print(sh.google_chrome("http://google.com"))


if __name__ == "__main__":
    # print(execute_command("ls"))
    # import sys
    #
    # defaultencoding = 'utf-8'
    # if sys.getdefaultencoding() != defaultencoding:
    #     os.reload(sys)
    #     sys.setdefaultencoding(defaultencoding)
    # test2()
    # test3()
    sh_test()




import json
# import file_util
import sys, os

curPath = os.path.abspath(os.path.dirname(__file__))
rootPath = os.path.split(curPath)[0]
sys.path.append(rootPath)
# print(sys.path)

from pylib import file_util


def json_load_byteified(file_handle):
    return _byteify(
        json.load(file_handle, object_hook=_byteify),
        ignore_dicts=True
    )


def json_loads_byteified(json_text):
    return _byteify(
        json.loads(json_text, object_hook=_byteify),
        ignore_dicts=True
    )


def _byteify(data, ignore_dicts=False):
    # if this is a unicode string, return its string representation
    if isinstance(data, unicode):
        return data.encode('utf-8')
    # if this is a list of values, return list of byteified values
    if isinstance(data, list):
        return [_byteify(item, ignore_dicts=True) for item in data]
    # if this is a dictionary, return dictionary of byteified keys and values
    # but only if we haven't already byteified it
    if isinstance(data, dict) and not ignore_dicts:
        return {
            _byteify(key, ignore_dicts=True): _byteify(value, ignore_dicts=True)
            for key, value in data.iteritems()
        }
        # if it's anything else, return it in its original form
    return data


# json格式化
def format_json(json_str):
    return format_obj(json.loads(json_str))


def format_obj(json_obj):
    return json.dumps(json_obj, indent=4, sort_keys=False, ensure_ascii=False)


def compress_json_file(file):
    content = file_util.read_content(file)
    jsn = json.loads(content)
    jstring = json.dumps(jsn)
    jstring = jstring.replace("\"", "\\\"")
    return jstring


def read_json_file(file):
    if os.path.exists(file):
        str = file_util.read_content(file)
        return json.loads(str)
    return {}


if __name__ == '__main__':
    import fire

    fire.Fire()



import random, types

from operator import itemgetter
from itertools import groupby


# 去除空字符元素
def removeEmpty(arr):
    while '' in arr:
        arr.remove('')
    return arr


# 合并数组
def merge_list(a, b):
    ret = []
    for i in a:
        ret.append(i)
    for i in b:
        ret.append(i)
    return ret


# 合并数组并排序
def merge_sort(a, b):
    ret = []
    while len(a) > 0 and len(b) > 0:
        if a[0] <= b[0]:
            ret.append(a[0])
            a.remove(a[0])
        if a[0] >= b[0]:
            ret.append(b[0])
            b.remove(b[0])
    if len(a) == 0:
        ret += b
    if len(b) == 0:
        ret += a
    return ret


# 递归的方法比较容易理解
def expand_list(nested_list=[[], []]):
    for item in nested_list:
        if isinstance(item, (list, tuple)):
            for sub_item in expand_list(item):
                yield sub_item
        else:
            yield item

            #
            # # 在stackoverflow看到大牛的列表生成式版本
            # func = lambda x: [y for l in x for y in func(l)] if type(x) is list else [x]


# 数组随机排序
def random_sort(list=[]):
    random.shuffle(list)
    return list


# 过滤数组
def filter_list(func, listdata=[]):
    return list(filter(func, listdata))


# 将大list按大小拆分为多个小list
def split_list_to_group(listdata=[], n=1):
    # 大列表中几个数据组成一个小列表
    # ret = []
    ret = [listdata[i:i + n] for i in range(0, len(listdata), n)]
    return ret


def foreach_list(func, list=[]):
    # print(type(func) is types.FunctionType)
    ret = []
    for i, value in enumerate(list):

        if func.__code__.co_argcount == 2:
            item = func(value, i)
        else:
            item = func(value)
        if item:
            ret.append(item)
    return ret


def foreach_dict(func, dict={}):
    ret = []
    if func.__code__.co_argcount == 3:
        for i, k in enumerate(dict.keys()):
            v = dict[k]
            item = func(k, v, i)
            if item:
                ret.append(item)
    else:
        for k, v in dict.items():
            item = func(k, v)
            if item:
                ret.append(item)
    return ret


def get_dict_keys(dic={}):
    ret = []
    for k, v in dic.items():
        ret.append(k)
    return ret


# 将内字典外列表，转为内列表外字典,每个key是一列
def convert_dict_list_to_list_dict(list=[]):
    dict = {}
    if len(list) > 0:
        dict = list[0]

    def append_dict(dict={}, item={}):
        keys = dict.keys()

        for k in keys:
            sublist = dict[k] if type(dict[k]).__name__ == 'list' else []

            sublist.append(item.get(k, None))
            dict[k] = sublist

    for i, item in enumerate(list):
        append_dict(dict, item)

    return dict


# 将内字典外列表，转为内列表外列表,二维表：内列表是一列，有多少个子列表就有多少行
def convert_dict_list_to_list_list(list=[], keys=[]):
    dic = convert_dict_list_to_list_dict(list)
    if len(keys) == 0:
        keys = dic.keys()

    ret = []
    for key in keys:
        ret.append(dic.get(key))

    return ret


# 将内列表外列表，转为内列表外字典
def convert_list_list_to_list_dict(list=[], keys=[]):
    dict = {}

    def append_dict(dict={}, item={}, keys=[]):

        for i, k in enumerate(keys):
            sublist = dict.get(k, [])

            sublist.append(item[i])
            dict[k] = sublist

            # break

    for i, item in enumerate(list):
        append_dict(dict, item, keys)
        # break

    return dict


# 将内列表外列表，转为内字典外列表
# 转换前：每个子列表为一列，有多少列就有几个子列表
# 转化后：每个子字典为一行，多个字典就是多行
def convert_columnlist_list_to_dict_list(list=[[]], keys=[]):
    ret = []

    row_count = len(list[0])

    for i in range(1, row_count):
        dict = {}
        for j, k in enumerate(keys):
            # print(j, k, len(list))
            dict[k] = list[j][i] if len(list) - 1 >= j else None
        ret.append(dict)

    return ret


def convert_list_to_tuple(list=[]):
    return tuple(list)


def convert_list_list_to_tuple_tuple(list=[]):
    ret = ()
    for item in list:
        print(ret)
        tp = tuple(item)
        if len(ret) == 0:
            ret = ((tp),)
        else:
            ret = ret + (tp,)
    return ret


# 过滤列表
def filter_list(judge_func, list=[]):
    return filter(judge_func, list)


# 按某个key取出字典列表中的值组成列表返回
def filter_dict_list_to_list(list=[], key=''):
    ret = []
    for item in list:
        obj = item.get(key, None)
        ret.append(obj)
    return ret


# 删除字典列表中字典的某个字段
def del_field_in_dict_list(list=[], key=''):
    ret = []
    for item in list:
        if key in item.keys():
            del item[key]
        ret.append(item)
    return ret


# 给字典数组加上一列相同的值
def add_dict_field_to_dict_list(list=[], key='', value=None):
    ret = []
    for item in list:
        item[key] = value
        ret.append(item)
    return ret


# # 给字典数组加上一列相同的值
# def fill_default_field_to_dict_list(list=[], key='', value=None):
#     ret = []
#     for item in list:
#         item[key] = value
#         ret.append(item)
#     return ret

# 将字典转成成数组
def convert_dict_to_list_list(dict={}):
    keys = []
    values = []
    for k, v in dict.items():
        keys.append(k)
        values.append(v)
    return [keys, values]


# 按key将列表分组
def groupByKey(rows=[{}], keyName=''):
    ret = {}
    for gp, items in groupby(rows, key=itemgetter(keyName)):
        # print(gp)
        for item in items:
            # print(item)
            ret[gp] = item
            break

    return ret


def merge_two_dicts(a, b):
    c = a.copy()  # make a copy of a
    c.update(b)  # modify keys and values of a with the ones from b
    return c


def merge_dictionaries(a, b):
    return {**a, **b}


def to_dict(keys=[], values=[]):
    return dict(zip(keys, values))


# spread([1,2,3,[4,5,6],[7],8,9]) # [1,2,3,4,5,6,7,8,9]
def spread(arg=[]):
    ret = []
    for i in arg:
        if isinstance(i, list):
            ret.extend(i)
        else:
            ret.append(i)
    return ret


if __name__ == '__main__':
    # a = [1, 3, 4, 6, 7, 78, 97, 190]
    # b = [2, 5, 6, 8, 10, 12, 14, 16, 18]
    #
    # print(merge_sort(a, b))
    # list = [{'a': 1, 'b': 2}, {'a': 1, 'b': 2}]
    # dict = convert_dict_list_to_list_dict(list)
    # print(dict)

    # l = [i for i in range(15)]
    # n = 3  # 大列表中几个数据组成一个小列表
    # ret = ([l[i:i + n] for i in range(0, len(l), n)])
    # print(ret)

    rows = [
        {'address': '5412 N CLARK', 'date': '07/01/2012'},
        {'address': '5148 N CLARK', 'date': '07/04/2012'},
        {'address': '5800 E 58TH', 'date': '07/02/2012'},
        {'address': '2122 N CLARK', 'date': '07/03/2012'},
        {'address': '5645 N RAVENSWOOD', 'date': '07/02/2012'},
        {'address': '1060 W ADDISON', 'date': '07/02/2012'},
        {'address': '4801 N BROADWAY', 'date': '07/01/2012'}]
    print(groupByKey(rows, 'date'))



import re, urllib
from pylib import request_util, url_util, log_util


def convert_curl_to_array(curl_str):
    header_arr = get_headrs(curl_str)
    # print(header_arr)

    # data_arr = get_data_str(curl_str)
    # print(data_arr)

    url = "'{0}'".format(get_url(curl_str))
    # print(url)
    headers = "-H '" + "' -H '".join(header_arr) + "'"
    # print(headers)
    data = " --data '" + get_data_str(curl_str) + "'"
    # print(data)

    return ['curl', url, headers, data, '--compressed']


# 获取get链接
def get_url(curl_str):
    pattern = re.compile(r'curl \'(.*?)\' ')
    url_arr = re.findall(pattern, curl_str)
    # 一般url不是在前面，就是在后面
    if len(url_arr) == 0:
        pattern = re.compile(r'--compressed \'(.*?)\'')
        url_arr = re.findall(pattern, curl_str)
    return ''.join(url_arr)


# 获取post参数字符串
def get_data_str(curl_str):
    pattern = re.compile(r'--data \'(.*?)\' ')
    data_arr = re.findall(pattern, curl_str)
    return ''.join(data_arr)


def get_data_binary_str(curl_str):
    pattern = re.compile(r'--data-binary \'(.*?)\' ')
    data_arr = re.findall(pattern, curl_str)
    return ''.join(data_arr)


def get_data_raw_str(curl_str):
    pattern = re.compile(r'--data-raw \'(.*?)\' ')
    data_arr = re.findall(pattern, curl_str)
    return ''.join(data_arr)


# 获取header数组
def get_headrs(curl_str):
    pattern = re.compile(r'-H \'(.*?)\' ')
    header_arr = re.findall(pattern, curl_str)
    return header_arr


# 获取header对象
def get_header_map(curl_str):
    header_arr = get_headrs(curl_str)
    heads = {}
    for item in header_arr:
        sub_arr = item.split(':', 1)
        if len(sub_arr) > 1:
            heads[sub_arr[0]] = sub_arr[1].strip()
    return heads


# 获取header值
def get_header_value(curl_str, key):
    mp = get_header_map(curl_str)
    return mp.get(key)


# 获取cookie字符串
def get_header_cookie_str(curl_str):
    headerMap = get_header_map(curl_str)
    cookie = headerMap.get('Cookie')
    if cookie == None:
        cookie = headerMap.get('cookie')
    return cookie


# 获取cookie内容
def get_cookie_dict(curl_str):
    cookiestr = get_header_cookie_str(curl_str)
    fields = cookiestr.split(';')
    # print(fields)

    dic = {}
    for f in fields:
        if f.find('=') > 0:
            key = f.split('=')[0].strip()
            dic[key] = f.split('=')[1].strip()

    # print(dic)
    return dic


# 获取post参数map
def get_param_map(curl_str):
    data_str = get_data_str(curl_str)
    arr = data_str.split('&')
    params = {}
    for param in arr:
        parr = param.split('=')
        if len(parr) > 1:
            params[parr[0]] = urllib.parse.unquote(parr[1])
    return params


# 用Request发出curl请求
def request_curl(curl_str):
    url = get_url(curl_str)
    # url = urllib.unquote(url)
    headers = get_header_map(curl_str)
    params = get_param_map(curl_str)
    if not params:
        params = get_data_binary_str(curl_str)

    if not params:
        params = get_data_raw_str(curl_str)

    # 如果还是没有参数，就发get请求
    if not params:
        gateway = url_util.get_url_gateway(url)
        params = url_util.get_url_params_map(url)
        urlparam = url_util.convert_dict_to_url_param(params)
        # return request_util.get(gateway+"?"+urlparam, params, headers)
        return request_util.get(gateway, params, headers)
    # print(url)
    # print(headers)
    # print(params)
    # log_util.loginfo("url: %s , headers: %s , params: %s", url, headers, params)

    return request_util.post(url, headers, params)


# 用Request发出curl请求
def request_download(curl_str, filepath):
    url = get_url(curl_str)
    # url = urllib.unquote(url)
    headers = get_header_map(curl_str)
    params = get_param_map(curl_str)
    if not params:
        params = get_data_binary_str(curl_str)

    # 如果还是没有参数，就发get请求
    if not params:
        gateway = url_util.get_url_gateway(url)
        params = url_util.get_url_params_map(url)
        return request_util.download(gateway, filepath, headers=headers, params=params)
    # print(url)
    # print(headers)
    # print(params)

    return request_util.download(url, filepath, headers=headers, params=params)


# 组装参数对
def build_headers_str(prefix, headers):
    arr = []
    for h in headers:
        str = "%s '%s'" % (prefix, h)
        arr.append(str)
    params_str = " ".join(arr)
    # print(params_str)
    return params_str


# 组装CURL字符串
def build_curl(url, headers, params):
    headers_str = build_headers_str('-H', headers)
    str_list = ['curl', "'%s'" % (url), headers_str, '--compressed']
    curl_str = " ".join(str_list)
    # print(curl_str)
    return curl_str


# 将参数值做urldecode
def urldecode_map_value(map={}):
    res = {}
    for k, v in map.items():
        # print(k, v)
        if isinstance(v, str):
            v = urllib.parse.unquote(v)
        res[k] = v
    return res


# 替换cookie
def replace_cookie(curl_str, cookie_new):
    cookie_new = replace_cookie_prefix(cookie_new)
    cookie_old = get_header_cookie_str(curl_str)
    return curl_str.replace(cookie_old, cookie_new)


# 替换data
def replace_data(curl_str, data_new):
    data_old = get_data_str(curl_str)
    if data_old == None or len(data_old.strip()) == 0:
        data_old = get_data_binary_str(curl_str)
    if data_old == None or len(data_old.strip()) == 0:
        data_old = get_data_raw_str(curl_str)
    return curl_str.replace(data_old, data_new)


def replace_cookie_prefix(cookie):
    cookie = cookie.replace("Cookie: ", "")
    cookie = cookie.replace("cookie: ", "")
    return cookie


# 替换机器地址，待http头
def replace_host(curl_str, host):
    chost = get_header_value(curl_str, 'Origin')
    curlstr = curl_str.replace(chost, host)
    return curlstr


# def replace_host(curl_str,)


if __name__ == '__main__':
    curl_str = '''
    curl 'http://icicada-zth-1.gz00b.stable.alipay.net/iworkbench/auto_name.json?_input_charset=utf-8&ctoken=SAf6vbiOqhtBGSs5&_ts=1583823348297' -H 'Connection: keep-alive' -H 'Accept: application/json, text/javascript, */*; q=0.01' -H 'X-Requested-With: XMLHttpRequest' -H 'User-Agent: Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.122 Safari/537.36' -H 'Content-Type: application/x-www-form-urlencoded; charset=UTF-8' -H 'Origin: http://icicada-zth-1.gz00b.stable.alipay.net' -H 'Referer: http://icicada-zth-1.gz00b.stable.alipay.net/iworkbench/VisitorCard.htm?appVersion=1.0001&loginEmail=youxi123_01@163.com' -H 'Accept-Language: zh-CN,zh;q=0.9' -H 'Cookie: JSESSIONID=2A292F2783B3230F3171B5AA87C04100; ; appExtInfo=%7B%22appVersion%22%3A%221.0001%22%2C%22appAlias%22%3A%22iworkbench%22%7D; BUSERVICE_TNTINSTID=ALIPW3SG; BUSERVICE_LAST_TNTINSTID=ALIPW3SG; zone=GZ00A; session.cookieNameId=ALIPAYBUMNGJSESSIONID; buservice_domain_id=KOUBEI_SALESCRM; SSO_EMPID_HASH_V2=17c3b8e74374497fc55cfc816bfb0afb; LOCALE=en_US; JSESSIONID=2A292F2783B3230F3171B5AA87C04100; route_group=LINKE_EI61574190_DFT_1070; ALIPAYIBUSERVICEJSESSIONID=GZ00F9012XD4fdWO0vXSngrex4TfWjipaybuserviceGZ00; ALIPAYBUMNGJSESSIONID=GZ00arb3Zwj5FtNXE2CW1yLf6VaX6yantbuserviceGZ00; ctoken=SAf6vbiOqhtBGSs5; BUSERVICE_SSO=4A21EF489ADA5128D2207262CB0CE6ED821817F331CA76B1C70F1779F4C59579; IAM_TOKEN=eyJraWQiOiJkZWZhdWx0IiwidHlwIjoiSldUIiwiYWxnIjoiUlMyNTYifQ.eyJjbmwiOiJCVUMiLCJzdWIiOiJqaW5sb25nLnJoaiIsImF1dGhfdHAiOlsiQ0VSVCIsIkRPTUFJTiJdLCJpc3MiOiJidW1uZy5zdGFibGUuYWxpcGF5Lm5ldCIsIm5vbmNlIjoiN2UxMzBlZmIiLCJzaWQiOiI5MDU0NCIsImF1ZCI6IioiLCJuYmYiOjE1ODM4MjMyODEsInNubyI6IjQwNDA2IiwidG50X2lkIjoiQUxJUFczQ04iLCJuYW1lIjoi6ZSm6b6ZIiwiZXhwIjoxNTgzOTA5NzQxLCJpYXQiOjE1ODM4MjMzNDEsImp0aSI6ImE3MGZmZWFmZjc5MDQ4MmVhZWI2MWNjODhjNzM0MmQyIn0.h-duunyJAbkyUNfs_ckMl_UOZYtgoMPd2akPetV_QrQ-ctgBZBCnSDmwzaJ3Umos5nsZAXIHILu5UNvT7-Wu8A' --data 'graphql=query%7BCrmMerchantQueryService_queryMerchantByLoginName_GraphQLString(arg0%3A%22youxi123_01%2540163.com%22)%7B*%7D%7D&appExtInfo=%7B%22appVersion%22%3A%221.0001%22%2C%22appName%22%3A%22iworkbench%22%2C%22appId%22%3A%221161901020000244043%22%2C%22tntInstId%22%3A%22ALIPW3SG%22%2C%22packageVersion%22%3A%228.0001%22%2C%22deployId%22%3A%221472003090000132225%22%2C%22appAlias%22%3A%22iworkbench%22%7D&uuid=83c2fd65-b7b4-44b6-bac2-f95d073ec15c' --compressed --insecure
    '''

    # convert_curl_to_array(curl_str)
    # print(get_param_map(curl_str))
    # print(get_header_map(curl_str))
    # url = get_url(curl_str)
    # print(url)

    # headers = get_headrs(curl_str)
    # params = get_param_map(curl_str)
    # build_headers_str('-H', headers)
    # build_curl(url, headers, params)
    # request_util.post(url, get_headrs(curl_str), get_param_map(curl_str))

    # curl_str = replace_cookie(curl_str, "xxxxxxxxxxxxxxxxxxxxxxxxxxx")
    # curl_str = replace_data(curl_str, "ccccccccccccccccccccccccccccc")
    # print(curl_str)
    # cookiestr = get_header_cookie_str(curl_str)
    # print(cookiestr)
    print(get_cookie_dict(curl_str))



# 判断变量类型的函数
def typeof(variate):
    type = None
    if isinstance(variate, int):
        type = "int"
    elif isinstance(variate, str):
        type = "str"
    elif isinstance(variate, float):
        type = "float"
    elif isinstance(variate, list):
        type = "list"
    elif isinstance(variate, tuple):
        type = "tuple"
    elif isinstance(variate, dict):
        type = "dict"
    elif isinstance(variate, set):
        type = "set"
    return type


# 返回变量类型
def getType(variate):
    arr = {"int": "整数", "float": "浮点", "str": "字符串", "list": "列表", "tuple": "元组", "dict": "字典", "set": "集合"}
    vartype = typeof(variate)
    if not (vartype in arr):
        return "未知类型"
    return arr[vartype]


def getFunctionByName(module_name, function_name):
    module = __import__(module_name)
    return getattr(module, function_name)


def print_func(obj):
    repr(obj)



import pymysql
import logging
import sys

# 加入日志
# 获取logger实例
logger = logging.getLogger("baseSpider")
# 指定输出格式
formatter = logging.Formatter('%(asctime)s%(levelname)-8s:%(message)s')
# 文件日志
file_handler = logging.FileHandler("baseSpider.log")
file_handler.setFormatter(formatter)
# 控制台日志
console_handler = logging.StreamHandler(sys.stdout)
console_handler.setFormatter(formatter)

# 为logge添加具体的日志处理器
logger.addHandler(file_handler)
logger.addHandler(console_handler)

logger.setLevel(logging.INFO)


class DBHelper:
    # 构造函数
    def __init__(self, host='127.0.0.1', user='root',
                 pwd='123456', db='testdb', port=3306):
        self.host = host
        self.user = user
        self.pwd = pwd
        self.db = db
        self.port = port
        self.conn = None
        # self.cur = None

    # 连接数据库
    def connectDatabase(self):
        try:
            self.conn = pymysql.connect(host=self.host, user=self.user, password=self.pwd,
                                        database=self.db, port=self.port, charset='utf8')
        except:
            logger.error("connectDatabase failed")
            return False
        self.cur = self.conn.cursor()
        return True

    # 关闭数据库
    def close(self):
        # 如果数据打开，则关闭；否则没有操作
        if self.conn and self.cur:
            self.cur.close()
            self.conn.close()
        return True




    def get_conn(self):
        """
        获取连接
        :rtype: object
        """
        return self.conn

    def close_conn(self):
        """
        关闭连接
        :rtype: object
        """
        try:
            if self.conn:
                self.conn.close()
        except pymysql.Error as e:
            print(e)

    def query(self, sql, close=True):
        """
        查询sql
        :rtype: object
        """
        self.connectDatabase()
        # 获取cursor
        cursor = self.conn.cursor()
        # 执行sql
        cursor.execute(sql)

        # 拿到结果
        # print(cursor.fetchall())
        res = [dict(zip([x[0] for x in cursor.description], row)) for row in cursor.fetchall()]
        # 关闭cursor
        cursor.close()
        if close:
            # 关闭连接
            self.close_conn()
        return res

    def execute(self, sql, commit=True, close=True):
        """
        执行sql
        :rtype: object
        """
        res = True
        self.connectDatabase()
        # 获取cursor
        cursor = self.conn.cursor()
        try:
            # 执行sql
            cursor.execute(sql)
            if commit:
                self.conn.commit()
        except pymysql.Error as e:
            self.conn.rollback()
            print(e)
            res = False
        # 关闭cursor
        cursor.close()
        if close:
            # 关闭连接
            self.close_conn()
        return res

    def find(self, table, where, field="*", close=True):
        """
        查询单条记录
        :rtype: object
        """
        sql = self.get_select_sql(table, where, field)
        # 获取cursor
        cursor = self.conn.cursor()
        try:
            # 执行sql
            cursor.execute(sql)
            # 拿到结果   dict(zip(['a','b'],(1,2)))  ==> {'a': 1, 'b': 2}  cursor.fetchone 拿到的是这条数据的元祖格式
            res = dict(zip([x[0] for x in cursor.description], cursor.fetchone()))
        except pymysql.Error as e:
            res=str(e)
        # 关闭cursor
        cursor.close()
        if close:
            # 关闭连接
            self.close_conn()
        return res

    def select(self, table, where, field="*", orderby="", page=0, page_size=10, close=True):
        """
        查询多条记录
        :rtype: object
        """
        sql = self.get_select_sql(table, where, field, orderby, page, page_size)
        return self.query(sql, close)

    def insert(self, table, dict, commit=True, close=True):
        """
        插入
        :rtype: object
        """
        sql = self.get_insert_sql(table, dict)
        return self.execute(sql, commit, close)

    def update(self, table, seting, where="", commit=True, close=True):
        """
        更新
        :rtype: object
        """
        sql = self.get_update_sql(table, seting, where)
        return self.execute(sql, commit, close)

    def delete(self, table, where, commit=True, close=True):
        """
        删除
        :rtype: object
        """
        where_string = self.get_where_sql(where)
        sql = "delete from {} {}".format(table, where_string)
        return self.execute(sql, commit, close)

    def get_select_sql(self, table, where="", field="*", orderby='', page=0, page_size=10):
        """
        获取查询sql语句
        :rtype: object
        """
        where_string = self.get_where_sql(where)
        if isinstance(field, list):
            field = ','.join(field)
        sql = "select {} from {} {}".format(field, table, where_string)
        if orderby != '':
            sql = sql.strip() + ' order by ' + orderby
        if page != 0:
            offset = (page - 1) * page_size
            sql = sql + " limit %s,%s" % (offset, page_size)
        return sql.strip() + ';'

    def get_insert_sql(self, table, dict):
        """
        获取插入sql语句
        :rtype: object
        """
        fields = '`' + '`,`'.join(dict.keys()) + '`'
        values = []
        for k in dict.keys():
            v = dict[k]
            if type(v) not in (int, float, str):
                if not v:
                    values.append('')
                else:
                    s = u'值错误, key: {}, value: {}, type: {}'.format(k, v, type(v))
                    raise Exception(s)
            if v == 'now()':
                values.append(v)
            elif type(v) is str:
                values.append("'{}'".format(v.replace('\\', '\\\\')))
            else:
                values.append('{}'.format(v))
        value_str = ','.join(values)
        return 'INSERT INTO {} ({}) VALUE ({});'.format(table, fields, value_str)

    def get_update_sql(self, table, seting, where):
        """
        获取更新sql语句
        :rtype: object
        """
        where_string = self.get_where_sql(where)
        if isinstance(seting, dict):
            tmp = []
            for k, v in seting.items():
                tmp.append(k + " = '" + str(v) + "'")
            set_string = ','.join(tmp)
        else:
            set_string = seting
        return 'update {} set {} {};'.format(table, set_string, where_string)

    def get_where_sql(self, where):
        """
        拼接where查询语句
        :rtype: object
        """
        if isinstance(where, dict):
            tmp = []
            for k, v in where.items():
                tmp.append("and " + str(k) + "='" + str(v) + "' ")
            if tmp != []:
                where_string = 'where' + ''.join(tmp).lstrip('and')
            else:
                where_string = ""
        else:
            where_string = where
        return where_string


if __name__ == '__main__':
    dbhelper = DBHelper()
    # 创建数据库的表
    sql = "create table maoyan('id'varchar(8),\
            'title'varchar(50),\
            'star'varchar(200), \
            'time'varchar(100),primary key('id'));"
    result = dbhelper.execute(sql)
    if result:
        logger.info("maoyan　table创建成功")
    else:
        logger.error("maoyan　table创建失败")



# 将上下文中的列表类型
def append_to_context_item_list(context={}, item="list", list_value=None):
    lst = context.get(item)
    lst = [] if lst == None else lst
    lst.append(list_value)
    context[item] = lst


# 返回上下文中的列表类型
def get_context_item_list(context={}, item="list"):
    lst = context.get(item)
    lst = [] if lst == None else lst
    return lst



import sys


class StringBuffer(object):
    def __init__(self):
        self.buffer = []

    def write(self, *args, **kwargs):
        self.buffer.append(args)


# 将函数的标准输出内容拦截并返回
def stdout_intercept_wrapper(func, *args, **kwargs):
    stdout = sys.stdout
    sys.stdout = StringBuffer()

    func(*args, **kwargs)

    sb, sys.stdout = sys.stdout, stdout
    # print(sb.buffer)
    ret = ""
    for item in sb.buffer:
        # print(item[0],end='')
        ret += item[0]
    # print(''.join(sb.buffer))
    return ret



import numpy as np
import pandas as pd
from pandas.core.frame import DataFrame


# 不错的教程 https://www.jianshu.com/p/6eb5499cd07d


# 将excel单sheet中的二维表读取到list的对象中
def read_excel_sheet(excel_path, sheet_name, titles=[], skiprows=0):
    ret = []
    df = pd.read_excel(excel_path, sheet_name=sheet_name, skiprows=skiprows)

    for index, row in df.iterrows():
        obj = {}
        if len(titles) == 0:
            titles = row.keys()
        for t in titles:
            obj[t] = row[t]
        ret.append(obj)
    return ret


# 返回整个excel所有sheet表格，resutl[sheet1]=[{a:1,b:2}]
def read_excel_sheets(excel_path, skiprows=0):
    ret = {}

    df = pd.read_excel(excel_path, sheet_name=None, skiprows=skiprows)
    # 遍历工作表
    for k in df.keys():
        list = []
        # print(df)
        for index, row in df[k].iterrows():
            obj = {}
            for t in row.keys():
                # print(t)
                obj[t] = row[t]
            list.append(obj)
        ret[k] = list
    return ret


# 将list[dict]对象转换为df
def convert_list_to_dataframe(list=[]):
    # 将 list 转为 dict
    # 将 dict 转为 Dataframe
    # 写入到 excel 里。
    fields = []
    if len(list) > 0:
        fields = list[0].keys()

    dict = {}
    # 初始化每个字段的列表
    for f in fields:
        dict[f] = []

    for item in list:
        for k in item.keys():
            # 给字典中填充列表数据
            dict[k].append(item[k])

    df = DataFrame(dict)
    print(df)
    return df


# 将df转换为list[[123],[223]]格式
def convert_dataframe_to_list_list(df):
    train_data = np.array(df)  # np.ndarray()
    # print(train_data)
    train_x_list = train_data.tolist()  # list
    return train_x_list


# 将df转换为list[{a:1},{a:2}]格式, headers 表示需要重新指定的标题列表
def convert_dataframe_to_dict_list(df, headers=[]):
    # dfname._stat_axis.values.tolist()  # 行名称
    # dfname.columns.values.tolist()  # 列名称

    header_list = df.columns.values.tolist() if len(headers) == 0 else headers
    # print(header_list)
    list_list = convert_dataframe_to_list_list(df)
    # print(list_list)

    # >>> d = {'a': 1, 'b': 2, 'c': 3, 'd': 4}
    # >>> d
    # {'a': 1, 'c': 3, 'b': 2, 'd': 4}
    # >>> d.keys()
    # ['a', 'c', 'b', 'd']
    # >>> d.values()
    # [1, 3, 2, 4]
    # >>> zip(d.values(),d.keys())
    # [(1, 'a'), (3, 'c'), (2, 'b'), (4, 'd')]
    # >>> dict(zip(d.values(),d.keys()))
    # {1: 'a', 2: 'b', 3: 'c', 4: 'd'}

    # 先遍历rows里面的值，然后在用zip反转
    dict_list = [dict(zip(header_list, row)) for row in list_list] if list_list else None
    # print(dict_list)
    return dict_list


# 将list[dict]对象写入excel单sheet表
def write_excel(list=[], excel_path=""):
    df = convert_list_to_dataframe(list)
    df.to_excel(excel_path)


# 将dict{list[dict]}对象写入excel多sheet表
def write_excel_sheets(dict={}, excel_path=""):
    write = pd.ExcelWriter(excel_path)

    fields = dict.keys()

    for (sheet, list) in dict.items():
        df = convert_list_to_dataframe(list)
        df.to_excel(write, sheet_name=sheet, header=fields, index=False)
    write.save()


# 将dataframe数据集写入到excel中
def write_df_to_excel(df, excel_path="", sheet_name='Sheet1'):
    writer = pd.ExcelWriter(excel_path)
    df.to_excel(writer, index=True, encoding='utf-8', sheet_name=sheet_name)
    writer.save()


# 将多个df合并追加
def append_dataframe(dst_df, source_df, ignore_index=False):
    if dst_df is None:
        dst_df = source_df
    else:
        dst_df = dst_df.append(source_df, ignore_index=ignore_index)
    return dst_df


def new_dataframe(column_names=[], rows_list=[]):
    df = pd.DataFrame(columns=column_names, data=rows_list)
    return df


def test():
    # 读取Excel的第一个表单

    df = pd.read_excel('demo1.xlsx')

    # 输出列标题

    print(df.columns.values)

    # 第第1行数据(不包含表头)

    data = df.loc[0].values

    print(data)

    # 第2~3行的数据

    data = df.loc[[1, 2]].values

    print(data)

    data = {"姓名": ["程序猿", "程旭阳", "陈续员"], "性别": ["男", "女", "未知"],

            "年龄": [50, 55, 60], "课程": ["Python", "学不懂", "学不动"]}

    df = pd.DataFrame(data)


if __name__ == '__main__':
    excel_path = "/Users/jinlong.rhj/Downloads/2019年2月班级人数表（含学籍号） - 副本.xlsx"
    res = read_excel_sheets(excel_path, 1)
    print(res['一年级'])
    write_excel(res['一年级'], '/Users/jinlong.rhj/Downloads/bbb.xlsx')
    # convert_list_to_dataframe(res['一年级'])
    write_excel_sheets(res, '/Users/jinlong.rhj/Downloads/ccc.xlsx')




    # # #按行求和
    # df['row_sum'] = df.apply(lambda x: x.sum(), axis=1)
    #
    # #按列求和
    # df.loc['col_sum'] = df.apply(lambda x: x.sum())



import time, functools, math, datetime, arrow

format_date_general = '%Y-%m-%d'
format_datetime_general = '%Y-%m-%d %H:%M:%S'


def getNowDateWithTimeZone():
    return arrow.now().format('YYYY-MM-DD HH:mm:ss ZZ')


def getCurDateStr():
    nowTime = time.localtime()
    year = str(nowTime.tm_year)
    month = str(nowTime.tm_mon)
    if len(month) < 2:
        month = '0' + month
    day = str(nowTime.tm_mday)
    if len(day) < 2:
        day = '0' + day
    return (year + '-' + month + '-' + day)


def getSimpleDateTimeStr():
    return time.strftime('%Y-%m-%d %H:%M:%S', time.localtime(time.time()))


# 获取10位长度的时间戳，精确到秒
def getTimestamps():
    return math.ceil(time.time())


# 将时间戳转换成时间,精确到秒
def convert_timestamp_to_date(timestamp, format='%Y-%m-%d %H:%M:%S'):
    # 转换成localtime
    time_local = time.localtime(timestamp)
    # 转换成新的时间格式(2016-05-05 20:28:54)
    dt = time.strftime(format, time_local)
    return dt


def convert_timestr_to_datetime(timestr, format='%Y-%m-%d %H:%M:%S'):
    # time_local = time.strptime(timestr, format)
    # 转换成新的时间格式(2016-05-05 20:28:54)
    # dt = time.strftime(format, time_local)

    # 转为时间数组，分别利用time模块和datetime模块
    # timeArray = time.strptime(timestr, format)
    # print(timeArray)

    datetimeArray = datetime.datetime.strptime(timestr, format)

    return datetimeArray


# 将datetime对象转为字符串
def datetime_to_str(date=None, format='%Y-%m-%d %H:%M:%S'):
    if date is None:
        date = datetime.datetime.now()
    return date.strftime(format)


def convert_to_timezone(dt=datetime.datetime.utcnow(), timezone_hour=8):
    print(dt)
    dt = dt.replace(tzinfo=datetime.timezone.utc)
    print(dt)
    tzutc = datetime.timezone(datetime.timedelta(hours=timezone_hour))
    local_dt = dt.astimezone(tzutc)
    print(local_dt)
    print(local_dt.strftime('%Y-%m-%d %H:%M:%S%z'))

    return local_dt


def add_day(start, n):
    # start = datetime.datetime.now()
    date = start + datetime.timedelta(days=n)
    return date


def add_hour(start, n):
    date = start + datetime.timedelta(hours=n)
    return date


def add_seconds(start, n):
    date = start + datetime.timedelta(seconds=n)
    return date


def timestr2sec(x):
    x = "00:00:" + x if len(x.split(':')) < 2 else x
    x = "00:" + x if len(x.split(':')) < 3 else x
    '''
    字符串时分秒转换成秒
    '''
    h, m, s = x.strip().split(':')  # .split()函数将其通过':'分隔开，.strip()函数用来除去空格
    return int(h) * 3600 + int(m) * 60 + int(s)  # int()函数转换成整数运算


def time2sec(y):
    '''
    时间类型时分秒转换成秒
    '''
    h = y.hour  # 直接用datetime.time模块内置的方法，得到时、分、秒
    m = y.minute
    s = y.second
    return int(h) * 3600 + int(m) * 60 + int(s)  # int()函数转换成整数运算


# 统计方法执行耗时
# @time_cost("cost")
# def test2(x):
#     time.sleep(0.2)
def time_cost(info="used"):
    def _time_cost(fn):
        @functools.wraps(fn)
        def _wrapper(*args, **kwargs):
            start = time.time()
            fn(*args, **kwargs)
            # print(time.time() - start)
            print("%s %s %s" % (fn.__name__, info, time.time() - start), "second")

        return _wrapper

    return _time_cost


class TimeCost(object):
    def __init__(self):
        pass

    def __call__(self, fn):
        def _call(*args, **kw):
            start = time.time()
            fn(*args, **kw)
            print("%s %s" % (fn.__name__, time.time() - start), "second")

        return _call


@time_cost()
def test(x, y):
    time.sleep(3)


@time_cost("cost")
def test2(x):
    time.sleep(1)


def sleep(secends):
    time.sleep(secends)


def countdown(secends):
    for i in range(secends, 0, -1):
        print("\r", "Please Wait: {}s".format(i), end="", flush=True)
        time.sleep(1)
    print("\r", "Please Wait: {}s".format(0), end="", flush=True)
    print()


if __name__ == '__main__':
    # test(1, 2)
    # test2(2)
    # print(getSimpleDateTimeStr())
    # print(getTimestamps())
    # print(convert_timestr_to_datetime('2015-05-27', format_date_general))
    # print(datetime.datetime.now())
    # convert_to_timezone()
    countdown(5)



import pymongo
from pymongo import MongoClient


# 获得集合连接
def collection(host, port=27017, db=None, user_name=None, password=None, col_name=None):
    client = pymongo.MongoClient(host=host, port=port)
    mdb = client[db]
    mdb.authenticate(user_name, password)
    col = mdb[col_name]
    return col


# 主键冲突插入不报错
def insertSilent(collection={}, query={}, data={}):
    collection.update(query, {'$setOnInsert': data}, True)


# 插入或更新
def insertOrUpdate(collection={}, query={}, insert={}, update={}):
    if insert:
        collection.update(query, {'$setOnInsert': insert}, True)
        # collection.update(query, {'$setOnInsert': insert, '$set': update}, True)
    else:
        collection.update(query, {'$set': update}, True)


# 创建索引
def create_index(col, field):
    col.create_index([(field, -1)], unique=True)


def find(col, query={}, ret_field=[], sort_by={}, limit=()):
    field = {}
    for item in ret_field:
        field[item] = 1
    result = col.find(query, field).sort([("created_at", pymongo.ASCENDING)]).limit(limit).skip(0)
    # result = self.fund.find({'type': '指数型'}, {'id': 1, 'fundSName': 1}).sort('total_assets', -1).limit(100)
    ret_list = list(result)
    return ret_list


# 将MongoDB的结果转换为list，不直接用list(result[:]),是因为直接list强转不能识别limit
def convert_result_list(result):
    list = []
    i = 0
    for item in result:
        i += 1
        # print(i, item)
        list.append(item)
    return list



import tablib
import os


def test():
    # 创建dataset，方法1
    dataset1 = tablib.Dataset()
    header1 = ('ID', 'Name', 'Tel', 'Age')
    dataset1.headers = header1
    dataset1.append([1, 'zhangsan', 13711111111, 16])
    dataset1.append([2, 'lisi', 13811111111, 18])
    dataset1.append([3, 'wangwu', 13911111111, 20])
    dataset1.append([4, 'zhaoliu', 15811111111, 25])
    print('dataset1:', os.linesep, dataset1, os.linesep)

    # 创建dataset，方法2
    header2 = ('ID', 'Name', 'Tel', 'Age')
    data2 = [
        [1, 'zhangsan', 13711111111, 16],
        [2, 'lisi', 13811111111, 18],
        [3, 'wangwu', 13911111111, 20],
        [4, 'zhaoliu', 15811111111, 25]
    ]
    dataset2 = tablib.Dataset(*data2, headers=header2)
    print('dataset2: ', os.linesep, dataset2, os.linesep)

    # 增加行
    dataset1.append([5, 'sunqi', 15911111111, 30])  # 添加到最后一行的下面
    dataset1.insert(0, [0, 'liuyi', 18211111111, 35])  # 在指定位置添加行
    print('增加行后的dataset1: ', os.linesep, dataset1, os.linesep)

    # 删除行
    dataset1.pop()  # 删除最后一行
    dataset1.lpop()  # 删除第一行
    del dataset1[0:2]  # 删除第[0,2)行数据
    print('删除行后的dataset1:', os.linesep, dataset1, os.linesep)

    # 增加列
    # 现在dataset1就剩两行数据了
    dataset1.append_col(('beijing', 'shenzhen'), header='city')  # 增加列到最后一列
    dataset1.insert_col(2, ('male', 'female'), header='sex')  # 在指定位置添加列
    print('增加列后的dataset1: ', os.linesep, dataset1, os.linesep)

    # 删除列
    del dataset1['Tel']
    print('删除列后的dataset1: ', os.linesep, dataset1, os.linesep)

    # 获取各种格式的数据
    print('yaml format: ', os.linesep, dataset1.yaml, os.linesep)
    print('csv format: ', os.linesep, dataset1.csv, os.linesep)
    print('tsv format: ', os.linesep, dataset1.tsv, os.linesep)

    # 然后就可以通过下面这种方式得到各种格式的数据了。
    # data.xlsx
    # data.xls
    # data.ods
    # data.json
    # data.yaml
    # data.csv
    # data.tsv
    # data.html
    # data.df     pandas

    # 导出到Excel表格中
    dataset1.title = 'dataset1'  # 设置Excel中表单的名称
    dataset2.title = 'dataset2'
    myfile = open('mydata.xlsx', 'wb')
    myfile.write(dataset1.xlsx)
    myfile.close()

    # 如果有多个sheet表单，使用DataBook就可以了
    myDataBook = tablib.Databook((dataset1, dataset2))
    myfile = open(myfile.name, 'wb')
    myfile.write(myDataBook.xlsx)
    myfile.close()


# 将list[{}]转为数据集
def dict_list_to_dataset(lst=[]):
    # 创建dataset，方法1
    dataset = tablib.Dataset()

    header = tuple(lst[0].keys())
    print(header)
    dataset.headers = header

    for dic in lst:
        values = []
        for k in header:
            values.append(dic.get(k))
        print(values)
        dataset.append(values)
    return dataset


def from_excel(file):
    # data = tablib.Dataset()
    # data.xls = open(file, 'rb').read()
    f = open(file, 'rb')
    data = tablib.import_set(f.read(), format='xlsx')
    return data


def to_excel(file, datasets=[]):
    # myfile = open(file, 'wb')
    # myfile.write(dataset1.xlsx)
    # myfile.close()

    # 如果有多个sheet表单，使用DataBook就可以了
    myDataBook = tablib.Databook(tuple(datasets))
    myfile = open(file, 'wb')
    myfile.write(myDataBook.xlsx)
    myfile.close()


def test2():
    from faker import Faker

    fake = Faker(locale='zh_CN')
    # 初始化

    print(fake.street_name())

    dic = {
        'name': fake.name(),
        'street': fake.street_name(),
    }
    lst = [dic, dic]
    print(lst)
    dataset = dict_list_to_dataset(lst)
    print(dataset.html)


if __name__ == '__main__':
    # test2()
    excel_path = '/Users/jinlong.rhj/商户管理服务数据.xlsx'
    ds = from_excel(excel_path)
    print(ds)



import requests, json
from pylib import url_util, object_util


def default_session():
    session = requests.Session()
    # print(1)
    # session.headers.update({'Cookie': 'JSESSIONID=083B61FCADA4DB211F33FDBE28B6F7DC; _gscu_1276227783=33815256qcspdf21; route=9822522b30f663ac278178ba378ac6d4'})
    session.headers.update({
        'User-Agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/68.0.3440.106 Safari/537.36'})
    return session


def build_session(heads={}):
    session = requests.Session()
    if any(heads):
        session.headers.update(heads)
    else:
        session = default_session()
    return session


# 解析url中的参数请求
def getFromUrl(url):
    u = url_util.get_url_gateway(url)
    p = url_util.get_url_params_map(url)
    ret = get(u, p)
    return ret


# 发送get请求，网关与参数要分开
def get(url, params={}, headers=None, payload=None, proxies={}, timeout=60):
    # session = default_session()
    # ret1 = session.get(url)
    # print(proxies)
    ret1 = requests.get(url, params, headers=headers, proxies=proxies, timeout=timeout, verify=False)
    return ret1


# 发送post请求
def post(url, headers=None, payload=None, proxies={}):
    # print(url)
    # print(headers)
    # print(payload)
    # session = build_session(headers)
    if object_util.typeof(payload) == 'str':
        payload = payload.encode("utf-8")
    elif object_util.typeof(payload) == 'dict':
        payload = json.loads(json.dumps(payload).encode('utf-8'))
    ret = requests.post(url, headers=headers, data=payload, proxies=proxies, verify=False)
    # print(ret)
    # print(ret.text)
    return ret


def post_json(url, json={}):
    return requests.post(url=url, json=json)


# 文件下载
def download(url, down_path, params={}, headers=None, payload=None, proxies={}, verify=True):
    r = requests.get(url, params, headers=headers, proxies=proxies, verify=verify)
    with open(down_path, "wb") as code:
        code.write(r.content)


# 转换返回内容的编码
def convert_res_charset(res, charset):
    return res.text.encode(res.encoding).decode(charset)


def get_json(url, params={}, headers=None, payload=None):
    ret = get(url, params, headers, payload)

    ret_json = {}
    if len(ret.text) > 0:
        ret_json = json.loads(ret.text)
    return ret_json


def get_proxies_param(proxyserver, proxyport):
    proxy = '%s:%s' % (proxyserver, proxyport)
    proxies = {
        'http': 'socks5://' + proxy,
        'https': 'socks5://' + proxy,
    }
    return proxies


if __name__ == '__main__':
    resp = get("https://www.facebook.com/", proxies=get_proxies_param('navyran.top', 38989))
    print(resp.text)



from setuptools import setup, find_packages
from codecs import open
from os import path


here = path.abspath(path.dirname(__file__))
setup(
    name='nylib',
    version='1.0',
    description='ny python lib utils',
    long_description=str(open(path.join(here, "ReadMe.md")).read()),
    # The project's main homepage.
    # url='nylib - Overview',
    # Author details
    author='navyran',
    author_email='navyran@gmail.com',
    # Choose your license
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: System :: Logging',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
    ],
    py_modules=["cmd_util"],
    install_requires=['requests', 'records'],
    include_package_data=True,
    package_data={'': ['*.yaml']},
    # packages=["."]
    packages=find_packages()
    # packages=find_packages(where='.', exclude=(), include=('*.py',))
)



from tqdm import trange
from rich.progress import track
import time


def tqdm_test():
    for i in trange(10):
        time.sleep(1)


def rich():
    for step in track(range(30)):
        print('早起Python')
        time.sleep(0.5)


if __name__ == '__main__':
    # rich()
    tqdm_test()


import xlrd, xlwt, openpyxl
from xlrd import xldate_as_tuple

from openpyxl import load_workbook
import datetime

'''
xlrd中单元格的数据类型
数字一律按浮点型输出，日期输出成一串小数，布尔型输出0或1，所以我们必须在程序中做判断处理转换
成我们想要的数据类型
0 empty,1 string, 2 number, 3 date, 4 boolean, 5 error
'''


class ExcelData():
    # 初始化方法
    def __init__(self, data_path, sheetname):
        # 定义一个属性接收文件路径
        self.data_path = data_path
        # 定义一个属性接收工作表名称
        self.sheetname = sheetname
        # 使用xlrd模块打开excel表读取数据
        self.data = xlrd.open_workbook(self.data_path)
        # 根据工作表的名称获取工作表中的内容（方式①）
        self.table = self.data.sheet_by_name(self.sheetname)
        # self.table,_ = self.get_sheet()
        # 根据工作表的索引获取工作表的内容（方式②）
        # self.table = self.data.sheet_by_name(0)
        # 获取第一行所有内容,如果括号中1就是第二行，这点跟列表索引类似
        self.keys = self.table.row_values(0) if self.table and self.table.nrows > 0 else []
        # 获取工作表的有效行数
        self.rowNum = self.table.nrows
        # 获取工作表的有效列数
        self.colNum = self.table.ncols

    # 定义一个读取excel表的方法
    def readExcel(self):
        # 定义一个空列表
        datas = []
        for i in range(1, self.rowNum):
            # 定义一个空字典
            sheet_data = {}
            for j in range(self.colNum):
                # 获取单元格数据类型
                c_type = self.table.cell(i, j).ctype
                # 获取单元格数据
                c_cell = self.table.cell_value(i, j)
                if c_type == 2 and c_cell % 1 == 0:  # 如果是整形
                    c_cell = int(c_cell)
                elif c_type == 3:
                    # 转成datetime对象
                    date = datetime.datetime(*xldate_as_tuple(c_cell, 0))
                    c_cell = date.strftime('%Y/%m/%d %H:%M:%S')
                elif c_type == 4:
                    c_cell = True if c_cell == 1 else False
                sheet_data[self.keys[j]] = c_cell
                # 循环每一个有效的单元格，将字段与值对应存储到字典中
                # 字典的key就是excel表中每列第一行的字段
                # sheet_data[self.keys[j]] = self.table.row_values(i)[j]
            # 再将字典追加到列表中
            datas.append(sheet_data)
        # 返回从excel中获取到的数据：以列表存字典的形式返回
        return datas

    def write_cell(self, row_num, col_num, value):
        sheet, wb = self.get_sheet()
        sheet.cell(row_num, col_num, value)  # 行，列，值 这里是从1开始计数的
        wb.save(self.data_path)  # 一定要保存

    def get_sheet(self):
        wb = load_workbook(filename=self.data_path)
        if not wb:
            wb = openpyxl.Workbook()  # 新建工作簿
        if self.sheetname not in wb.sheetnames:
            wb.create_sheet(self.sheetname)  # 添加页
            # sheet = wb.active  # 获得当前活跃的工作页，默认为第一个工作页
        sheet = wb.get_sheet_by_name(self.sheetname)  # 获得指定名称页
        return sheet, wb

    def delete_sheet(self):
        ws, wb = self.get_sheet()
        wb.remove(ws)
        wb.save(self.data_path)

    def delete_row(self, row_num):
        wb = load_workbook(filename=self.data_path)
        sheet = wb.get_sheet_by_name(self.sheetname)  # 获得指定名称页
        for row in range(row_num, sheet.max_row):
            for column in range(sheet.max_column):
                sheet[row][column].value = sheet[row + 1][column].value
        for cell in list(sheet.rows)[sheet.max_row - 1]:
            cell.value = None
        wb.save(self.data_path)

    # col_num是从1开始，不是从0开始
    def delete_column(self, col_num):
        wb = load_workbook(filename=self.data_path)
        sheet = wb.get_sheet_by_name(self.sheetname)  # 获得指定名称页
        sheet.delete_cols(col_num)
        wb.save(self.data_path)

    # 'A1:G37'
    def delete_mult_cell(self, area='A1:A1'):
        ws, wb = self.get_sheet()
        for row in ws[area]:
            for cell in row:
                cell.value = None
        wb.save(self.data_path)

    # 追加一行，values为字符串数组
    def append_row(self, values):
        sheet, wb = self.get_sheet()
        # items = [i for i in values]
        # for row in range(0, len(values)):
        #     sheet.append([row])
        sheet.append(values)
        wb.save(self.data_path)

    # 写入列
    def write_column(self, values=[], col_num=0):
        sheet, wb = self.get_sheet()
        rows = sheet.max_row
        columns = sheet.max_column
        col_num = columns + 1 if col_num <= 0 else col_num
        # column_data = []
        assert len(values) == rows, 'values length: %s must equal with rows length: %s' % (len(values), rows)
        for i in range(1, rows + 1):
            sheet.cell(i, col_num, values[i - 1])  # 行，列，值 这里是从1开始计数的
        wb.save(self.data_path)

    # 获取表格的总行数和总列数
    def get_row_col_num(self):
        sheet, wb = self.get_sheet()
        rows = sheet.max_row
        columns = sheet.max_column
        return rows, columns

    # 获取某个单元格的值
    def get_cell_value(self, row, column):
        sheet, wb = self.get_sheet()
        cell_value = sheet.cell(row=row, column=column).value
        return cell_value

    # 获取某列的所有值
    def get_col_value(self, column):
        sheet, wb = self.get_sheet()
        rows = sheet.max_row
        column_data = []
        for i in range(1, rows + 1):
            cell_value = sheet.cell(row=i, column=column).value
            column_data.append(cell_value)
        return column_data

    # 获取某列的所有值
    def get_col_obj(self, column):
        sheet, wb = self.get_sheet()
        rows = sheet.max_row
        column_data = []
        for i in range(1, rows + 1):
            cell_value = sheet.cell(row=i, column=column)
            column_data.append(cell_value)
        return column_data

    # 获取列超链接
    def get_col_hyberlink(self, column):
        col = excel.get_col_obj(column)
        ret = []
        for c in col:
            link = None
            if c.hyperlink:
                # print(c.hyperlink.display)
                link = c.hyperlink.display
            ret.append(link)
        return ret

    # 获取某行所有值
    def get_row_value(self, row):
        sheet, wb = self.get_sheet()
        columns = sheet.max_column
        row_data = []
        for i in range(1, columns + 1):
            cell_value = sheet.cell(row=row, column=i).value
            row_data.append(cell_value)
        return row_data

    def copy_sheet_from(self, fiel_src, ws_src_name):
        wb_src = load_workbook(filename=fiel_src)
        ws_src = wb_src[ws_src_name]

        sheet, wb = self.get_sheet()

        # 两个for循环遍历整个excel的单元格内容
        for i, row in enumerate(ws_src.iter_rows()):
            for j, cell in enumerate(row):
                sheet.cell(row=i + 1, column=j + 1, value=cell.value)

        wb.save(self.data_path)


def read_to_dict_list(data_path, sheetname='Sheet1'):
    get_data = ExcelData(data_path, sheetname)
    datas = get_data.readExcel()
    return datas


if __name__ == "__main__":
    data_path = "/Users/jinlong.rhj/Book1.xlsx"
    sheetname = "Sheet1"
    excel = ExcelData(data_path, sheetname)
    # datas = get_data.readExcel()
    # print(datas)
    links = excel.get_col_hyberlink(2)
    excel.write_column(links)



import time
from pytz import utc
from apscheduler.jobstores.redis import RedisJobStore
from apscheduler.schedulers.blocking import BlockingScheduler
from apscheduler.triggers.cron import CronTrigger
from apscheduler.schedulers.background import BackgroundScheduler
from apscheduler.jobstores.mongodb import MongoDBJobStore
from apscheduler.jobstores.sqlalchemy import SQLAlchemyJobStore
from apscheduler.executors.pool import ThreadPoolExecutor, ProcessPoolExecutor
import functools

import redis  # 导入redis模块，通过python操作redis 也可以直接在redis主机的服务端操作缓存数据库

blockingScheduler = BlockingScheduler()
backgroundScheduler = BackgroundScheduler()

scheduler = None

# pool = redis.ConnectionPool(
#     host='docker-jl.dockerlab.alipay.net',
#     port=6379,
#     password='redisjinlong'
# )
# rds = redis.StrictRedis(connection_pool=pool)

jobstores = {
    'redis': RedisJobStore(2, host='docker-jl.dockerlab.alipay.net',
                           port=6379,
                           password='redisjinlong'),
    'mongo': MongoDBJobStore(
        host='mongodb://test:testjinlong@mongo-jl.dockerlab.alipay.net:27017/apscheduler?authMechanism=SCRAM-SHA-1&authSource=apscheduler',
        port=27017, ),
    'default': SQLAlchemyJobStore(url='sqlite:///jobs.sqlite')
}

executors = {

    'default': ThreadPoolExecutor(20),
    'processpool': ProcessPoolExecutor(5)
}

job_defaults = {

    'coalesce': False,
    'max_instances': 3
}


def initScheduler(init_scheduler=None):
    global scheduler
    if init_scheduler != None:
        scheduler = init_scheduler
        return scheduler
    else:
        scheduler = blockingScheduler

    return scheduler


# 常规阻塞间隔执行，单位为秒
def run_blocking_scheduler_interval(job_func, seconds):
    scheduler = initScheduler()
    job = scheduler.add_job(job_func, 'interval', seconds=seconds)
    scheduler.start()


# def run_blocking_scheduler_datetime(job_func,datetime):


def redis_scheduler_interval(job_func, seconds):
    scheduler = initScheduler()
    scheduler.add_jobstore('redis', jobs_key='example.jobs', run_times_key='example.run_times')
    scheduler.start()


# 直接运行的方法
def run_scheduler_cron(job_func, cron):
    scheduler = initScheduler()
    # print(scheduler)
    # sched.add_job(job_function, CronTrigger.from_crontab('0 0 1-15 may-aug *'))
    job = scheduler.add_job(job_func, CronTrigger.from_crontab(cron))
    scheduler.start()


def run_pool():
    backgroundScheduler = BackgroundScheduler(jobstores=jobstores, executors=executors, job_defaults=job_defaults,
                                              timezone=utc)
    scheduler = initScheduler(backgroundScheduler)
    job = scheduler.add_job(job_func, trigger='interval', seconds=2, jobstore='redis', id='aaac')
    scheduler.start()
    return scheduler


# cron运行装饰器, 被装饰的方法不能有入参
def run_cron_decorator(cron):
    '''定时任务装饰器'''

    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kw):
            try:
                run_scheduler_cron(func, cron)
            except Exception as e:
                print(e)
                pass

        return wrapper

    return decorator


# 间隔运行装饰器, 被装饰的方法不能有入参
def run_interval(interval_seconds=10):
    '''定时任务装饰器'''

    def decorator(func):
        @functools.wraps(func)
        def wrapper(*args, **kw):
            try:
                run_blocking_scheduler_interval(func, interval_seconds)
            except Exception as e:
                print(e)
                pass

        return wrapper

    return decorator


def evaluate(expression):
    '''
    order of values
    year, month, day, week, day_of_week, hour, minute, second, start_date, end_date, timezone
    '''
    splitValues = expression.split()
    for i in range(0, 8):
        if (i == 0):
            if (splitValues[0] == '?'):
                year = None
            else:
                year = splitValues[0]
        if (i == 1):
            if (splitValues[1] == '?'):
                month = None
            else:
                month = splitValues[1]
        if (i == 2):
            if (splitValues[2] == '?'):
                day = None
            else:
                day = splitValues[2]
        if (i == 3):
            if (splitValues[3] == '?'):
                week = None
            else:
                week = splitValues[3]
        if (i == 4):
            if (splitValues[4] == '?'):
                day_of_week = None
            else:
                day_of_week = splitValues[4]
        if (i == 5):
            if (splitValues[5] == '?'):
                hour = None
            else:
                hour = splitValues[5]
        if (i == 6):
            if (splitValues[6] == '?'):
                minute = None
            else:
                minute = splitValues[6]
        if (i == 7):
            if (splitValues[7] == '?'):
                second = None
            else:
                second = splitValues[7]
    return year, month, day, week, day_of_week, hour, minute, second


def getTrigger(cronExpression):
    year, month, day, week, day_of_week, hour, minute, second = evaluate(cronExpression)
    trigger = CronTrigger(year, month, day, week, day_of_week, hour, minute, second)
    return trigger


def job_func():
    print(111)


@run_cron_decorator('*/1 * * * *')
def job_func2():
    print(111)


if __name__ == '__main__':
    # redis_scheduler_interval(job_func, 10)
    # test_redis()
    # scheduler = run_pool()
    # time.sleep(10)
    # scheduler.remove_job('aaac', 'redis')
    run_blocking_scheduler_interval(job_func, 1)
    cron = '*/1 * * * *'
    # print(getTrigger(cron))
    # run_scheduler_cron(job_func, cron)
    # job_func2()



from logging import exception


class Stack():
    def __init__(self, size):
        self.size = size
        self.stack = []
        self.top = -1

    def push(self, ele):  # 入栈之前检查栈是否已满
        if self.isfull():
            raise exception("out of range")
        else:
            self.stack.append(ele)
            self.top = self.top + 1

    def pop(self):  # 出栈之前检查栈是否为空
        if self.isempty():
            raise exception("stack is empty")
        else:
            self.top = self.top - 1
            return self.stack.pop()

    def isfull(self):
        return self.top + 1 == self.size

    def isempty(self):
        return self.top == -1
